/*
 * Request.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.http;

import java.util.*;

/**
 *
 * @author  lars
 */
public class Request 
{
  Hashtable headers = new Hashtable();
  
  String uri = "/";
  String querystring = "";
  String body;
  
  String method = "POST";
  
  com.rimfaxe.balancer.VirtualHost virtualhost;
  com.rimfaxe.balancer.RealServer realhost;
  
  String host = "";
  Vector cookielist = new Vector();
  
  
  /** Creates a new instance of Request */
  public Request() 
  {   
  }
  
  public void setRequestBody(String body)
  {
    this.body = body;
  }
  
  public void setQuerystring(String query)
  {
    this.querystring = query;
  }
  
  public void setURI(String uri)
  {
    this.uri = uri;
  }
  
  public void setMethod(String m)
  {
    this.method = m;
  }
  
  public void setVirtualHost(com.rimfaxe.balancer.VirtualHost host)
  {
    this.virtualhost = host;
  }
  
  public void setRealHost(com.rimfaxe.balancer.RealServer host)
  {
    this.realhost = host;
  }
  
  public void addHeader(String header, String val) 
  {
    //com.rimfaxe.util.Log.log("Request Header ["+header+"] ["+val+"]");   
    if (header.equalsIgnoreCase("cookie")) 
    {
      com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(val,"; ",false);
      while (tkz.hasMoreTokens())
      {
        String tmp = tkz.nextToken();
        //System.out.println("Add Cookie : "+tmp);
        Cookie cookie = new Cookie(tmp);
        cookielist.addElement(cookie);
      }
    }
    if (header.equalsIgnoreCase("rimfaxe_loadbalancer_uri"))
    {
      this.uri = val;
    }
    if (header.equalsIgnoreCase("host"))
    {
      this.host = val;
    }
    headers.put(header,val);
  }
  
  
  public Vector getCookies() { return this.cookielist; }
  
  public Cookie getCookie(String name) 
  {
    Cookie c = null;  
    Enumeration enum = cookielist.elements();
    while ( (c==null) && (enum.hasMoreElements()) )
    {
      Cookie cookie = (Cookie) enum.nextElement();
      if (cookie.getName().equalsIgnoreCase(name))
      c = cookie;
    }
    return c;
  }
  
  public com.rimfaxe.balancer.VirtualHost getVirtualHost() { return this.virtualhost; }
  
  public com.rimfaxe.balancer.RealServer getRealHost() { return this.realhost; }
  
  public String getURI() { return this.uri; }
  
  public String getHost() { return this.host; }
  
  public String getRequestBody() { return this.body; }
  
  public String getMethod() { return this.method; }
  
  public String getHeader(String header)
  {
    return (String) headers.get(header);
  }
  
 
  public Enumeration getHeaderKeys()
  {
    return headers.keys();  
  }
  
  
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("com.rimfaxe.http.Request\n\n");
    buf.append("Real Host = "+realhost+"\n");
    buf.append("Virtual Host = "+virtualhost+"\n");
    buf.append("URI = "+uri+"\n");
    buf.append("Query String = "+querystring+"\n");
    buf.append("\n\nHeaders:\n\n");
    Enumeration enum = getHeaderKeys();
    while (enum.hasMoreElements())
    {
      String header = (String) enum.nextElement();  
      buf.append("["+header+"]  ["+getHeader(header)+"]\n");
    }
    buf.append("\n\nRequest Body (POST parameters): \n\n"+body+"\n\n");
    return ""+buf;
  }
     
}
