/*
 * Response.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.http;

import java.util.*;
import java.io.*;


/**
 *
 * @author  lars
 */
public class Response 
{
  Cookie cookie;  
  Vector headers = new Vector();

  String content = "";
  String contenttype = "text/plain";
  String encoding = "binary";
  
  String httpline = "HTTP/1.1 200 OK";
  String location;
  
  Integer errorcode = new Integer(0);
  
  InputStream stream = null;  
  
  /** Creates a new instance of Response */
  public Response(com.rimfaxe.http.Request request, InputStream is) 
  {  
     //BufferedReader d = new BufferedReader(new InputStreamReader(is));
     
     System.out.println("Create response from input stream");
     try
     {
       httpline = readLine(is);
       System.out.println("http line -> "+httpline);
       parseErrorCode(httpline);
     
       String line = readLine(is);
     
       while ( !line.equalsIgnoreCase("")  )
       {
            if (line==null)
            {
              return;  
            }
    
            StringTokenizer tk2 = new StringTokenizer(line,":",true);
            String name = tk2.nextToken().trim();
            tk2.nextToken();
            String val = tk2.nextToken("\n").trim();
            
            boolean addheader = true;
            
            if (name.equalsIgnoreCase("Content-Type")) { contenttype = val.trim(); }
            if (name.equalsIgnoreCase("Location")) 
            {
               location = processLocation(request, val.trim());
               val = location;
            }
            
            if (name.equalsIgnoreCase("Set-Cookie"))  
            { 
               cookie = new Cookie(val.trim()); 
            }
            
            if (name.equalsIgnoreCase("Connection"))  { addheader = false;  }
            
            if (addheader) 
            {
              System.out.println("Adding header in response : "+name+"->"+val);  
              headers.addElement( new Header(name,val) );
            }
            line = readLine(is);
        
       }
       
       stream = is;
     }
     catch (IOException ex) 
     {
       // TODO
     }
  }
  
  
  /** Creates a new instance of Response */
  public Response(com.rimfaxe.http.Request request, InputStream is, java.net.HttpURLConnection uc) 
  {  
     
     try
     {
       errorcode = new Integer(  uc.getResponseCode()  );
       
       //System.out.println("Error code = "+errorcode);
       
       boolean headers_done = false;
       int idx = 0;
     
       while ( !headers_done  )
       {
           idx++; 
           String name = uc.getHeaderFieldKey(idx); 
           
           if (name==null)
           {
             
               headers_done = true;  
           }
           else
           {
              String val = uc.getHeaderField(idx);    
              boolean addheader = true;
              if (name.equalsIgnoreCase("Content-Type")) 
              { 
                contenttype = val.trim(); 
              }
              //if (name.equalsIgnoreCase("Location")) 
              //{
              //  location = processLocation(request, val.trim());
              //  val = location;
              //}
              if (name.equalsIgnoreCase("Set-Cookie"))  
              { 
                cookie = new Cookie(val.trim()); 
              }
              if (name.equalsIgnoreCase("Connection"))  { addheader = false;  }
              if (addheader) 
              {
                //if (errorcode.intValue()!=200)
                //  System.out.println("Adding header["+idx+"] in response : "+name+"->"+val);  
                headers.addElement( new Header(name,val) );
              }
              
           }
       }
    
      
       stream = is;
     }
     catch (IOException ex) 
     {
       ex.printStackTrace(System.out);  
     }
  }
  
  
  /** Creates a new instance of Response */
  public Response(com.rimfaxe.http.Request request, String raw) 
  {  
     
     
     StringBuffer buf = new StringBuffer();
     
     com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(raw,"\n\r",true);
     httpline = tkz.nextToken();
     
     parseErrorCode(httpline);
     
     int countLinefeeds  =  0;
     
     while ( (tkz.hasMoreTokens()) && (countLinefeeds<4) )
     {
        String line = tkz.nextToken();  
     
        if (line.equalsIgnoreCase("\n")) countLinefeeds++;
        else if (line.equalsIgnoreCase("\r")) countLinefeeds++;
        else
        {
            //System.out.println("Parsing line in response : "+line);  
            countLinefeeds = 0;
            com.rimfaxe.util.RimfaxeStringTokenizer tk2 = new com.rimfaxe.util.RimfaxeStringTokenizer(line,":",true);
            String name = tk2.nextToken().trim();
            tk2.nextToken();
            String val = tk2.nextToken("\n").trim();
            
            boolean addheader = true;
            
            if (name.equalsIgnoreCase("Content-Type")) 
            {
               //com.rimfaxe.util.Log.log(""+name+": " + val); 
               contenttype = val.trim();
            }
            if (name.equalsIgnoreCase("Location")) 
            {
               //com.rimfaxe.util.Log.log(""+name+": " + val); 
               location = processLocation(request, val.trim());
               val = location;
            }
            if (name.equalsIgnoreCase("Set-Cookie")) 
            {
              //System.out.println("Adding header in response : "+name+"->"+val); 
              cookie = new Cookie(val.trim());
            }
            if (name.equalsIgnoreCase("Connection")) 
            {
              addheader = false;   
              // do nothing! 
            }
            
            if (addheader) 
            {
              //System.out.println("Adding header in response : "+name+"->"+val);  
              headers.addElement( new Header(name,val) );
            }
        }
     } 
     
     while (tkz.hasMoreTokens())
     {
       buf.append( tkz.nextToken() );  
     }
     
     content = ""+buf;  
  }
  
  /** Creates a new instance of Response */
  public Response(String raw, boolean redirect) 
  {  
     StringBuffer buf = new StringBuffer();
     
     com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(raw,"\n\r",true);
     httpline = tkz.nextToken();
     
     parseErrorCode(httpline);
     
     int countLinefeeds  =  0;
     
     while ( (tkz.hasMoreTokens()) && (countLinefeeds<4) )
     {
        String line = tkz.nextToken();  
     
        if (line.equalsIgnoreCase("\n")) countLinefeeds++;
        else if (line.equalsIgnoreCase("\r")) countLinefeeds++;
        else
        {
            countLinefeeds = 0;
            com.rimfaxe.util.RimfaxeStringTokenizer tk2 = new com.rimfaxe.util.RimfaxeStringTokenizer(line,":",true);
            String name = tk2.nextToken().trim();
            tk2.nextToken();
            String val = tk2.nextToken("\n").trim();
            
            boolean addheader = true;
            
            if (name.equalsIgnoreCase("Content-Type")) 
            {
               //com.rimfaxe.util.Log.log(""+name+": " + val); 
               contenttype = val.trim();
            }
            if (name.equalsIgnoreCase("Location")) 
            {
               //com.rimfaxe.util.Log.log(""+name+": " + val); 
               //location = processLocation(request, val.trim());
               val = location;
            }
            if (name.equalsIgnoreCase("Set-Cookie")) 
            {
              //com.rimfaxe.util.Log.log("Set-Cookie :"+val);  
              cookie = new Cookie(val.trim());
            }
            if (name.equalsIgnoreCase("Connection")) 
            {
              addheader = false;   
              // do nothing! 
            }
            
            if (addheader) 
            {
              //System.out.println("Adding header in response : "+name+"->"+val);  
              headers.addElement( new Header(name,val) );
            }
        }
     } 
     
     while (tkz.hasMoreTokens())
     {
       buf.append( tkz.nextToken() );  
     }
     
     content = ""+buf;  
  }
  
  public Vector getHeaders() { return headers; }
  public Cookie getCookie() { return cookie; }
  
  public InputStream getStream() { return stream; }
  
  public String getContentType() { return contenttype; }
  public String getLocation() { return location; }
  public Integer getErrorCode() { return errorcode; }
    
  private void parseErrorCode(String line)
  {
    //System.out.println("Parse error code from http line :"+ line);  
    com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(line," ",false);
    String http = tkz.nextToken();
    String error = tkz.nextToken();
    //System.out.println("Status :"+ error);
    errorcode = new Integer(error);
  }
  
  public String getContent() 
  { 
    StringBuffer buf = new StringBuffer();
    
    //System.out.println("Content type = ["+getContentType()+"]");
    /*if (getContentType().startsWith("text/html")) buf.append("<!-- \n");
    if (getContentType().startsWith("text/html"))buf.append(""+httpline+"\r\n");
    Enumeration hl = headers.elements();
    while (hl.hasMoreElements())
    {
      com.rimfaxe.http.Header header = (com.rimfaxe.http.Header) hl.nextElement();
      String name = header.getName(); 
      String val = header.getValue();
      if (getContentType().startsWith("text/html")) buf.append(name+": "+val+"\r\n");  
    }
    if (getContentType().startsWith("text/html")) buf.append("\r\n");
    if (getContentType().startsWith("text/html")) buf.append("-->\n");
      */  
    buf.append(""+content);
    
    return ""+buf; 
  }
  
  
  public String processLocation(com.rimfaxe.http.Request request, String loc)
  {
    com.rimfaxe.balancer.RealServer rh = request.getRealHost();
    
    if (rh==null) return loc;
    
    String ver1 = "http://"+rh.getName();
    String ver2 = "http://"+rh.getName()+":"+rh.getPort();
    String ver3 = "http://"+rh.getIP();
    String ver4 = "http://"+rh.getIP()+":"+rh.getPort();
     
    boolean replace = false;
    int chars = 0;
    
    if (loc.startsWith(ver4))
    {
      replace = true;
      chars = ver4.length();
    }
    else if (loc.startsWith(ver3))
    {
      replace = true;
      chars = ver3.length();
    }
    else if (loc.startsWith(ver2))
    {
      replace = true;
      chars = ver2.length();
    }
    else if (loc.startsWith(ver1))
    {
      replace = true;
      chars = ver1.length();
    }
    
    String newloc = "http://"+request.getHost()+loc.substring(chars);
    
    //System.out.println("new location -> "+newloc);
    
    return newloc;
  }
  
  
  public String readLine(InputStream is)
      throws IOException
  {
     ByteArrayOutputStream baos = new ByteArrayOutputStream();
     
     byte[] buf = new byte[1];
     
     while (true)
     {
       int read = is.read(buf);
       if (read==0) return baos.toString();
       if (buf[0]=='\r') {}
       else if (buf[0]=='\n') { return baos.toString(); }
       else { baos.write(buf); }
     }
  }
}
