package com.rimfaxe.thread;

import java.io.StringWriter;
import java.io.PrintWriter;
import java.util.HashMap;
import java.io.OutputStream;
import java.io.PrintStream;

/** Default base class for all exceptions.
 */

public class ReportingException extends Exception
{
    /** The nested exception */
    private ReportingException mNestedException;

    /** The name value pair Hashmap */
    private HashMap mHashMap = null; // name value pair hashmap


    /** Constructs a ReportingException with message of the value passed through
     * the constructor.
     * @aMsg The msg string passed parameter is the message
     * assigned to this ReportingException.
     */

    public ReportingException(String aMessage) {
        super(aMessage);
    }


    /** the put method inserts name value pairs into the given exception. These values
     * should be passed parameters within the method and also other values of interest.
     * This method nests ReportingExceptions and stores other Exception values as their stack trace.
     * @param aName is a name String
     * @param aValue is a value String
     */

    public synchronized void put(String aName, Object aValue) {
        if (aValue instanceof ReportingException) mNestedException = (ReportingException) aValue;
        else if (aValue instanceof Exception) {
            mHashMap.put(aName, getStackTraceOf((Exception)aValue));
        } else
        mHashMap.put(aName, aValue);
    }



    /** the printStackTrace method outputs the stack trace to System.out
     *  this overrides the method printStackTrace from Exception.
     */

    public void printStackTrace()
    {
        //printStackTrace((OutputStream)System.out);
    }


    /** the printStackTrace method outputs the stack trace to aOutputStream
     *  this overrides the method printStackTrace from Exception.
     *  @param aKey string.
     */

    public void printStackTrace(OutputStream aOutputStream)
    {
        PrintStream lPrintStream = new PrintStream(aOutputStream);
        lPrintStream.print(getStackTrace());
    }


    /** the getTrace method returns a String value containing
     *  the stack trace for this exception. This does not use getStackTrace
     *  which is now implemented in Java 1.4
     *  @return String containing stack trace.
     */


    public String getTrace()
    {
        StringBuffer lBuffer = new StringBuffer();

        lBuffer.append("\n\n------------------------\n");
        lBuffer.append(this.getMessage());
        lBuffer.append("\n------------------------\n\n");
        if (mNestedException == null) { // this is the end of the tree -- print the stack trace information
            lBuffer.append(getStackTraceOf(this));
        } else {

            if (!mHashMap.isEmpty()) {
                lBuffer.append("\n-- Name Value Pairs --\n");
            lBuffer.append(mHashMap.toString()); }
            else lBuffer.append("\n-------------------\n");
            lBuffer.append(mNestedException.getStackTrace()); // recursive call to nested exception
        }

        return lBuffer.toString();
    }



    /** Gets the stored nested exception.
     *  This method is very useful to the ExceptionHandler which needs to know the entire stack of exceptions.
     *   @return nested exception */
    public Throwable getNestedException()
    {
        return mNestedException;
    }

    /** Utility method to get the stack trace string of the specified throwable. */
    private String getStackTraceOf(Throwable aThrowable)
    {
        if (aThrowable == null)
        return null;

        StringWriter s = new StringWriter();
        aThrowable.printStackTrace(new PrintWriter(s));
        return s.toString();
    }
}
