package com.rimfaxe.thread;

import java.util.*;

/** The ThreadWatchDog class is the watchdog used to monitor threads
 * in an application. There are two dogs used to monitor, the ALPHA_DOG
 * and the BETA_DOG. The BETA_DOG's sole responsibility is to monitor
 * the ALPHA_DOG. The ALPHA_DOG monitors the BETA_DOG and all assigned
 * threads for monitoring. If a thread dies, the dogs will report an
 * exception. If the dead thread is a MutableThread, the dog will try to
 * restart it.
 */
public class ThreadWatchDog extends MutableThread {

    private static ThreadWatchDog mAlphaThread = null; // alphadog instance
    private static ThreadWatchDog mBetaThread = null; // betadog instance
    private static HashMap mThreadHashMap = null; // vector of all threads
    private static int mAlphaPriority = Thread.NORM_PRIORITY;
    private static int mBetaPriority =  Thread.NORM_PRIORITY;
    private static long POLL_TIME = 3000; //  3 second check for TEST Change this to 60000 for one min. etc
    /** The attribute mWatchDogPollCount is added for test interest. It shows
     * how many times the polling has occurred within the dog.
     */
    public  int mWatchDogPollCount = 0; // test count
    private int mDogIdentity = 0;

    private ThreadWatchDog() 
    {
        super();
        mThreadHashMap = new HashMap();
    }

    public void endWatching()
    {
       mAlphaThread.clear();
       mBetaThread.clear();  
    }
    
    public void clear()
    {
      mThreadHashMap = new HashMap();  
    }
    
    /** The getInstance method insures that the ALPHA_DOG (and therefore also
     * the BETA_DOG) is a singleton. It is static so that it can be referenced
     * throughout the application within the JVM.
     */
    public synchronized static ThreadWatchDog getInstance()
    {
        if (mAlphaThread == null) {
            mAlphaThread = new ThreadWatchDog();
            mAlphaThread.setPriority(mAlphaPriority);
            mAlphaThread.setName("ALPHA DOG");
            mBetaThread = new ThreadWatchDog();
            mBetaThread.setPriority(mBetaPriority);
            mBetaThread.setName("BETA DOG");
            // these dogs watch each other
            mAlphaThread.put(mBetaThread);
            mBetaThread.put(mAlphaThread);
        }
        return mAlphaThread;
    }


    /** This is the run method for the thread.
     */
    public void run() 
    {
        //System.out.println("run()");
        while(true) 
        {

            try 
            {
                //System.out.println("checkAllThreads()");
                checkAllThreads();
                poll();

            } catch (ReportingException eReportingException) {
                ReportingExceptionHandler.processException(eReportingException);
            }
        }
    }


    /** The checkAllThreads method for a dog checks each thread which has been
     * put into the HashMap.
     */

     public synchronized void checkAllThreads()  {

        Set lThreadSet = mThreadHashMap.keySet();
        Iterator lThreadIterator = lThreadSet.iterator();

        while(lThreadIterator.hasNext())
        {
            String lThreadKey = (String) lThreadIterator.next();
            Object lTestThread =  mThreadHashMap.get(lThreadKey);

            if (lTestThread instanceof MutableThread)
            {

                if (!((MutableThread)lTestThread).isAlive())
                {
                    ReportingExceptionHandler.processException( new ReportingException("Mutable Thread " + lThreadKey + " is dead"));
                    try {
                        ReportingLogger.logDebug("Attempting to restart thread : " + lThreadKey);
                        // attempt to restart the thread by clearing and restarting
                        ReportingLogger.logDebug("Creating Thread for " + lThreadKey);
                        ((MutableThread)lTestThread).createThread();
                        ReportingLogger.logDebug("Starting " + lThreadKey);
                        ((MutableThread)lTestThread).start();
                        ReportingLogger.logDebug("Restart Successful");
                    } catch (Exception eException) {
                        ReportingException lReportingException = new ReportingException("Thread " + lThreadKey + " restart failure");
                        lReportingException.put("eException",eException);
                        ReportingExceptionHandler.processException(lReportingException);
                    }
                }
                else {
                    ReportingLogger.logDebug("Mutable Thread " + ((MutableThread)lTestThread).getName() + "  is Alive");
                }

                ReportingLogger.logDebug("Thread " + ((MutableThread)lTestThread).getName() + " is alive ");
                }  else  { // not a Mutable thread
                if (!((Thread)lTestThread).isAlive())
                {
                    
                    
                    // LOG this except for "BETA DOG" and "ALPHA DOG"
                    
                    
                    if (  (!lThreadKey.equalsIgnoreCase("ALPHA DOG"))  &&    (!lThreadKey.equalsIgnoreCase("BETA DOG"))  )
                    {
                       com.rimfaxe.util.Log.log("Thread " + lThreadKey + " is being restarted");   
                    }
                    
                    
                    
                    ReportingExceptionHandler.processException(new ReportingException("Thread " + lThreadKey + " is dead"));
                }
                else {
                    // thread is alive
                    ReportingLogger.logDebug("Thread " + ((Thread)lTestThread).getName() + "  is Alive");
                }
            }
        }

    }



    /** The put method puts a mutable thread into the HashMap for monitoring by a dog.
     * Note that this is an overloaded method which also allows for non-mutable Threads to
     * be monitored as well.
     */
    public synchronized void put(MutableThread aMutableThread) {
        ReportingLogger.logDebug("ThreadWatchDog.adding mutable thread : " + aMutableThread.getName());
        mThreadHashMap.put(aMutableThread.getName(), aMutableThread);
    }
    
    public synchronized void remove(MutableThread aMutableThread) 
    {
        ReportingLogger.logDebug("ThreadWatchDog.adding mutable thread : " + aMutableThread.getName());
        mThreadHashMap.remove(aMutableThread.getName());
    }

    /** The put method causes the dog to monitor the passed Thread.
     */
    public synchronized void put(Thread aThread) {
        ReportingLogger.logDebug("ThreadWatchDog.adding thread : " + aThread.getName());
        mThreadHashMap.put(aThread.getName(), aThread);
    }


    private synchronized void poll() throws ReportingException
    {
        try 
        {
            mWatchDogPollCount++; // for test and monitoring, shows the number of times polling has occurred
            ReportingLogger.logDebug("WatchDogPollCount is " + mWatchDogPollCount + " for " + this.getName());
            this.wait(POLL_TIME);
        } 
        catch (InterruptedException eInterruptedException) 
        {
            ReportingException lReportingException = new ReportingException("Interrupted Exception Thread WatchDog poll");
            lReportingException.put("eInterruptedException", eInterruptedException);
            throw lReportingException;
        }

    }
    
}


