/*
 * Calendar.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.util;

import java.util.*;

/**
 *
 * @author  Lars Andersen
 * @version 
 */
public class Calendar extends GregorianCalendar 
{
   
  public static TimeZone TZ = TimeZone.getTimeZone("Europe/Copenhagen");  

  /** Creates new calendar */
  public Calendar() 
  {
    super( com.rimfaxe.util.Calendar.TZ, Locale.ENGLISH);
  }
  
  /** Creates new calendar */
  public Calendar(int timezone) 
  {
    this.setTimeZone( new SimpleTimeZone(timezone,"CUSTOM") );  
  }
  
  /** Creates new calendar */
  public Calendar(int timezone, long millis) 
  {
    this.setTimeZone( new SimpleTimeZone(timezone,"CUSTOM") ); 
    this.setTimeInMillis(millis);
  }
    /** Creates new calendar */
  public Calendar(String in) 
  {
    System.out.println("NOTE -> Not compatible with GCJ Calendar(String)");  
    this.setTime( java.sql.Timestamp.valueOf(in) );  
  }
  
  /** Creates new calendar */
  public Calendar(String in, String format) 
  {
    try
    {
      if (format.equalsIgnoreCase("sql"))
      {
    
        StringTokenizer tkz = new StringTokenizer(in," ",false);
        String datestr = tkz.nextToken();
        String timestr = tkz.nextToken();
    
        tkz = new StringTokenizer(datestr,"-",false);
        String y = tkz.nextToken();
        String m = tkz.nextToken();
        String d = tkz.nextToken();
     
        Integer year = new Integer(y); this.set(YEAR, year.intValue());
        Integer month = new Integer(m); this.set(MONTH, month.intValue()-1);
        Integer day = new Integer(d); this.set(this.DATE, day.intValue());
      
        tkz = new StringTokenizer(timestr,":",false);
        String hour_s = tkz.nextToken();
        String minute_s = tkz.nextToken();
        String second_s = tkz.nextToken();
     
        Integer hour = new Integer(hour_s); this.set(HOUR_OF_DAY, hour.intValue());
        Integer minute = new Integer(minute_s); this.set(MINUTE, minute.intValue());
        Integer second = new Integer(second_s); this.set(SECOND, second.intValue());
      
      }
    }
    catch (Exception e)
    {
      // TODO
    }
  }
  

  public Calendar(java.sql.Timestamp ts)
  {
    this.setTime(ts);
  }

  public Calendar(java.sql.Date ts)
  {
    this.setTime(ts);
  }

  public Calendar(java.util.Date ts)
  {
    this.setTime(ts);
  }
  
  public static void setDefaultTimeZone(TimeZone tz)
  {
    TZ = tz;  
  }

  public void setSqlTimestamp(java.sql.Timestamp ts)
  {
    this.setTime(ts);
  }

  public void setSqlDate(java.sql.Date ts)
  {
    this.setTime(ts);
  }

  public long getTimeInMillisSpecial()
  {
    return this.getTimeInMillis();
  }
  
  // It should look like this :       Sun, 06 Nov 1994 08:49:37 GMT
  public String getHttpDate()
  {
    StringBuffer sb = new StringBuffer();
    switch(this.get(this.DAY_OF_WEEK)) 
    {
      case Calendar.MONDAY    : sb.append("Mon");    break;
      case Calendar.TUESDAY   : sb.append("Tue");   break;
      case Calendar.WEDNESDAY : sb.append("Wed");     break;
      case Calendar.THURSDAY  : sb.append("Thu");     break;
      case Calendar.FRIDAY    : sb.append("Fri");       break;
      case Calendar.SATURDAY  : sb.append("Sat");      break;
      case Calendar.SUNDAY    : sb.append("Sun");      break;
    }
    sb.append(", ");
    
    String year = "2000";
    String day = "01";

    year = "" + this.get(this.YEAR);
    int d = this.get(this.DATE);
    if (d<10) day = "0" + d; else day = "" + d;
    
    sb.append(day+" ");
    
    switch(this.get(this.MONTH)) {
      case Calendar.JANUARY  : sb.append("Jan");    break;
      case Calendar.FEBRUARY : sb.append("Feb");   break;
      case Calendar.MARCH    : sb.append("Mar");     break;
      case Calendar.APRIL    : sb.append("Apr");     break;
      case Calendar.MAY      : sb.append("May");       break;
      case Calendar.JUNE     : sb.append("Jun");      break;
      case Calendar.JULY     : sb.append("Jul");      break;
      case Calendar.AUGUST   : sb.append("Aug");    break;
      case Calendar.SEPTEMBER: sb.append("Sep"); break;
      case Calendar.OCTOBER  : sb.append("Oct");   break;
      case Calendar.NOVEMBER : sb.append("Nov");  break;
      case Calendar.DECEMBER : sb.append("Dec");  break;
    }
    sb.append(" "+year+" "+getPrettyTime()+" GMT");
    
    return ""+sb;
  }
  
  // It should look like this :       Sun, 06-03-1994 08:49:37 GMT
  public String getCookieDate()
  {
    StringBuffer sb = new StringBuffer();
    switch(this.get(this.DAY_OF_WEEK)) 
    {
      case Calendar.MONDAY    : sb.append("Mon");    break;
      case Calendar.TUESDAY   : sb.append("Tue");   break;
      case Calendar.WEDNESDAY : sb.append("Wed");     break;
      case Calendar.THURSDAY  : sb.append("Thu");     break;
      case Calendar.FRIDAY    : sb.append("Fri");       break;
      case Calendar.SATURDAY  : sb.append("Sat");      break;
      case Calendar.SUNDAY    : sb.append("Sun");      break;
    }
    sb.append(", ");
    
    String year = "2000";
    String day = "01";
    String month = "01";
    year = "" + this.get(this.YEAR);
    int d = this.get(this.DATE);
    if (d<10) day = "0" + d; else day = "" + d;
    int m = this.get(this.MONTH);
    if (m<10) month = "0" + m; else month = "" + m;
    
    sb.append(day+":"+month+":");
    
    
    sb.append(""+year+" "+getPrettyTime()+" GMT");
    
    return ""+sb;
  }
  
  // It should look like this :       Sun, 06/03 1994 08:49
  public String getPrintDateUK()
  {
    StringBuffer sb = new StringBuffer();
    switch(this.get(this.DAY_OF_WEEK)) 
    {
      case Calendar.MONDAY    : sb.append("Mon");    break;
      case Calendar.TUESDAY   : sb.append("Tue");   break;
      case Calendar.WEDNESDAY : sb.append("Wed");     break;
      case Calendar.THURSDAY  : sb.append("Thu");     break;
      case Calendar.FRIDAY    : sb.append("Fri");       break;
      case Calendar.SATURDAY  : sb.append("Sat");      break;
      case Calendar.SUNDAY    : sb.append("Sun");      break;
    }
    sb.append(", ");
    
    String year = "2000";
    String day = "01";
    String month = "01";
    year = "" + this.get(this.YEAR);
    int d = this.get(this.DATE);
    if (d<10) day = "0" + d; else day = "" + d;
    int m = this.get(this.MONTH)+1;
    if (m<10) month = "0" + m; else month = "" + m;
    
    sb.append(day+"/"+month+" ");
    
    
    sb.append(""+year+" "+getPrettyTime2());
    
    return ""+sb;
  }
  
  public String getTimestamp()
  {
    String year = "2000";
    String month = "01";
    String day = "01";

    year = "" + this.get(this.YEAR);
    int m = this.get(this.MONTH) + 1;
    if (m<10) month = "0" + m; else month = "" + m;
    int d = this.get(this.DATE);
    if (d<10) day = "0" + d; else day = "" + d;

    return new String(""+year+"-"
                           +month+"-"
                           +day+" "
                           +this.getPrettyTime());
  }
  
  public String getSqlTimestamp()
  {
    int month = this.get(this.MONTH) + 1;
    return new String("TIMESTAMP'"+this.get(this.YEAR)+"-"
                                    +month+"-"
                                    +this.get(this.DATE)+" "
                                    +this.get(this.HOUR_OF_DAY)+":"
                                    +this.get(this.MINUTE)+":"
                                    +this.get(this.SECOND)+"'");
  }

  public String getSqlDate()
  {
    String year = "2000";
    String month = "01";
    String day = "01";

    year = "" + this.get(this.YEAR);
    int m = this.get(this.MONTH) + 1;
    if (m<10) month = "0" + m; else month = "" + m;
    int d = this.get(this.DATE);
    if (d<10) day = "0" + d; else day = "" + d;
    
    return new String("'"+year+"-"+month+"-"+day+"'");
  }

  public String getSqlSmalldatetime()
  {
    String year = "2000";
    String month = "01";
    String day = "01";

    year = "" + this.get(this.YEAR);
    int m = this.get(this.MONTH) + 1;
    if (m<10) month = "0" + m; else month = "" + m;
    int d = this.get(this.DATE);
    if (d<10) day = "0" + d; else day = "" + d;

    return new String("'"+year+"-"
                           +month+"-"
                           +day+" "
                           +this.getPrettyTime()+"'");
  }
  
  

  

  public String getMonthName(int m)
  {
    switch(m) 
    {
      case Calendar.JANUARY  : return "januar";   
      case Calendar.FEBRUARY : return "februar";   
      case Calendar.MARCH    : return "marts";    
      case Calendar.APRIL    : return "april";   
      case Calendar.MAY      : return "maj";       
      case Calendar.JUNE     : return "juni";    
      case Calendar.JULY     : return "juli";     
      case Calendar.AUGUST   : return "august";   
      case Calendar.SEPTEMBER: return "september"; 
      case Calendar.OCTOBER  : return "oktober";   
      case Calendar.NOVEMBER : return "november"; 
      case Calendar.DECEMBER : return "december"; 
    }
    return "januar";
  }
  
  public int getMonthValue(String m)
  {
    if (m.equalsIgnoreCase("januar")) { return Calendar.JANUARY; } 
    else if (m.equalsIgnoreCase("februar")) { return Calendar.FEBRUARY; }
    else if (m.equalsIgnoreCase("marts")) { return Calendar.MARCH; }
    else if (m.equalsIgnoreCase("april")) { return Calendar.APRIL; }
    else if (m.equalsIgnoreCase("maj")) { return Calendar.MAY; }
    else if (m.equalsIgnoreCase("juni")) { return Calendar.JUNE; }
    else if (m.equalsIgnoreCase("juli")) { return Calendar.JULY; }
    else if (m.equalsIgnoreCase("august")) { return Calendar.AUGUST; }
    else if (m.equalsIgnoreCase("september")) { return Calendar.SEPTEMBER; }
    else if (m.equalsIgnoreCase("oktober")) { return Calendar.OCTOBER; }
    else if (m.equalsIgnoreCase("november")) { return Calendar.NOVEMBER; }
    else if (m.equalsIgnoreCase("december")) { return Calendar.DECEMBER; }
    else { return Calendar.JANUARY; }
  }
  
  public String getPrintDateDK()
  {
    StringBuffer sb = new StringBuffer();

    sb.append(""+this.get(this.DATE));
    sb.append(". ");
    switch(this.get(this.MONTH)) {
      case Calendar.JANUARY  : sb.append("januar ");    break;
      case Calendar.FEBRUARY : sb.append("februar ");   break;
      case Calendar.MARCH    : sb.append("marts ");     break;
      case Calendar.APRIL    : sb.append("april ");     break;
      case Calendar.MAY      : sb.append("maj ");       break;
      case Calendar.JUNE     : sb.append("juni ");      break;
      case Calendar.JULY     : sb.append("juli ");      break;
      case Calendar.AUGUST   : sb.append("august ");    break;
      case Calendar.SEPTEMBER: sb.append("september "); break;
      case Calendar.OCTOBER  : sb.append("oktober ");   break;
      case Calendar.NOVEMBER : sb.append("november ");  break;
      case Calendar.DECEMBER : sb.append("december ");  break;
    }
    sb.append(this.get(this.YEAR));
    return sb.toString();
  }

  public String getPrintDateSimpleDK() {
    java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("dd.MM.yy");
    return sdf.format(this.getTime());

  }

  public String getWeek()
  {
    StringBuffer buf = new StringBuffer();
    int w = getWeekNumber();
    if (w<10) buf.append("0"+w);
    else buf.append(""+w);
    return buf.toString();
  }

  
  public String getPrettyTime()
  {
    StringBuffer buf = new StringBuffer();
    
    int hour = this.get(this.HOUR_OF_DAY);
    int minute = this.get(this.MINUTE);
    int second = this.get(this.SECOND);
    
    if (hour<10) 
      buf.append("0"+hour);
    else
      buf.append(""+hour);
    
    buf.append(":");
    
    if (minute<10) 
      buf.append("0"+minute);
    else
      buf.append(""+minute);
    
    buf.append(":");
    
    if (second<10) 
      buf.append("0"+second);
    else
      buf.append(""+second);
    
    return buf.toString();
  }
  
  public String getPrettyTime2()
  {
    StringBuffer buf = new StringBuffer();
    
    int hour = this.get(this.HOUR_OF_DAY);
    int minute = this.get(this.MINUTE);
    //int second = this.get(this.SECOND);
    
    if (hour<10) 
      buf.append("0"+hour);
    else
      buf.append(""+hour);
    
    buf.append(":");
    
    if (minute<10) 
      buf.append("0"+minute);
    else
      buf.append(""+minute);
    
    //buf.append(":");
    
    //if (second<10) 
    //  buf.append("0"+second);
    //else
    //   buf.append(""+second);
    
    return buf.toString();
  }
  
  public String getYear()
  {
    int y = this.get(this.YEAR);
    return ""+y;
  }

  public String getMonth()
  {
    StringBuffer buf = new StringBuffer();
    int m = this.get(this.MONTH)+1;
    if (m<10) buf.append("0"+m);
    else buf.append(""+m);
    return buf.toString();
  }

  public String getDate()
  {
    StringBuffer buf = new StringBuffer();
    int d = this.get(this.DATE);
    if (d<10) buf.append("0"+d);
    else buf.append(""+d);
    return buf.toString();
  }

  public int getWeekNumber()
  {
    com.rimfaxe.util.Calendar x = new com.rimfaxe.util.Calendar();
    x.set(x.YEAR,this.get(this.YEAR));
    x.set(x.MONTH,this.get(this.MONTH));
    x.set(x.DATE,this.get(this.DATE));

    if (x.get(x.DAY_OF_WEEK)==1) x.add(x.WEEK_OF_YEAR,-1);

    x.set(x.DAY_OF_WEEK,5);

    int count = 1;
    boolean sameyear = true;
    int oldyear = x.get(x.YEAR);

    while (sameyear)
    {
      x.add(x.WEEK_OF_YEAR,-1);
      if (x.get(x.YEAR)==oldyear)
        count++;
      else
        sameyear = false;
    }

    return count;
  }
}
