/*
 * HuffmanCoder.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.util;

import java.util.*;

/**
 *
 * @author  Lars Andersen
 */
public class HuffmanCoder extends Object 
{
  private HashMap  _inverseMap = null;
  private String[] _code = null;
  

  private final static String[] map = {
    "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P",
    "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "a", "b", "c", "d", "e", "f",
    "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v",
    "w", "x", "y", "z", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "+", "/",
    
    ":A", ":B", ":C", ":D", ":E", ":F", ":G", ":H", ":I", ":J", ":K", ":L", ":M", ":N", ":O", ":P",
    ":Q", ":R", ":S", ":T", ":U", ":V", ":W", ":X", ":Y", ":Z", ":a", ":b", ":c", ":d", ":e", ":f",
    ":g", ":h", ":i", ":j", ":k", ":l", ":m", ":n", ":o", ":p", ":q", ":r", ":s", ":t", ":u", ":v",
    ":w", ":x", ":y", ":z", ":0", ":1", ":2", ":3", ":4", ":5", ":6", ":7", ":8", ":9", ":+", ":/",
    
    ";A", ";B", ";C", ";D", ";E", ";F", ";G", ";H", ";I", ";J", ";K", ";L", ";M", ";N", ";O", ";P",
    ";Q", ";R", ";S", ";T", ";U", ";V", ";W", ";X", ";Y", ";Z", ";a", ";b", ";c", ";d", ";e", ";f",
    ";g", ";h", ";i", ";j", ";k", ";l", ";m", ";n", ";o", ";p", ";q", ";r", ";s", ";t", ";u", ";v",
    ";w", ";x", ";y", ";z", ";0", ";1", ";2", ";3", ";4", ";5", ";6", ";7", ";8", ";9", ";+", ";/",
    
    "-A", "-B", "-C", "-D", "-E", "-F", "-G", "-H", "-I", "-J", "-K", "-L", "-M", "-N", "-O", "-P",
    "-Q", "-R", "-S", "-T", "-U", "-V", "-W", "-X", "-Y", "-Z", "-a", "-b", "-c", "-d", "-e", "-f",
    "-g", "-h", "-i", "-j", "-k", "-l", "-m", "-n", "-o", "-p", "-q", "-r", "-s", "-t", "-u", "-v",
    "-w", "-x", "-y", "-z", "-0", "-1", "-2", "-3", "-4", "-5", "-6", "-7", "-8", "-9", "-+", "-/"
  };
    
  class Ranking implements Comparable
  {
    private int   _value;
    private long  _frequency;

    public Ranking(int value, long frequency) 
    {
      _value = value;
      _frequency = frequency;
    }

    public int compareTo(Object o) 
    {
      Ranking other = (Ranking) o;
      if (other.getFrequency() < _frequency) 
      {
        return -1;
      } 
      else if (other.getFrequency() == _frequency) 
      {
        return 0;
      } 
      else 
      {
        return 1;
      }
    }

    public int getValue() { return _value; }
    public long getFrequency() { return _frequency; }
  }
  
  /** Creates new HuffmanCoder */
  public HuffmanCoder() 
  {}

  public String getMapping() 
  {
    StringBuffer res = new StringBuffer();
    
    Set set = _inverseMap.keySet();
    Iterator iter = set.iterator();
    while (iter.hasNext()) 
    {
      String key = ""+iter.next();
      res.append(""+key+","+_inverseMap.get(key)+",");
    }
    return ""+res;  
  }
  
  public void createMap(String mapping)
  {
    _inverseMap = new HashMap();  
    StringTokenizer tkz = new StringTokenizer(mapping.trim(),",",false);
    
    while (tkz.hasMoreTokens())
    {
      String key = tkz.nextToken();
      Integer val = new Integer( tkz.nextToken()  );
      _inverseMap.put(key,val);
    }
  }
  
  public byte[] decodeHuffman(String stream, String mapping)
  {
    createMap(mapping);   
    int index=0;
    int streamLength=stream.length();
    ArrayList list = new ArrayList();
    while (index < streamLength) 
    {
      char indexChar = stream.charAt(index);
      String code = null;
      if (indexChar == ':' || indexChar == ';' || indexChar == '-') {
        code = stream.substring(index, index+2);
        index += 2;
      } else {
        code = stream.substring(index, index+1);
        index += 1;
      }
      Integer value = (Integer) _inverseMap.get(code);
      list.add(value);
    }
    byte buffer[] = new byte[list.size()];
    Iterator iter = list.iterator();
    index = 0;
    while (iter.hasNext()) {
      buffer[index++] = ((Integer) iter.next()).byteValue();
    }
    return buffer;
  }
  
  public byte[] decodeHuffman(String stream, boolean std)
  {
    buildInverseMap();
    int index=0;
    int streamLength=stream.length();
    ArrayList list = new ArrayList();
    while (index < streamLength) 
    {
      char indexChar = stream.charAt(index);
      String code = null;
      if (indexChar == ':' || indexChar == ';' || indexChar == '-') {
        code = stream.substring(index, index+2);
        index += 2;
      } else {
        code = stream.substring(index, index+1);
        index += 1;
      }
      Integer value = (Integer) _inverseMap.get(code);
      list.add(value);
    }
    byte buffer[] = new byte[list.size()];
    Iterator iter = list.iterator();
    index = 0;
    while (iter.hasNext()) {
      buffer[index++] = ((Integer) iter.next()).byteValue();
    }
    return buffer;
  }
  
  public String encodeHuffman(byte[] buffer) 
  {
    StringBuffer encodedString = new StringBuffer();

      int readBytes = buffer.length;
      long[] freqCount = new long[256];
      
      for (int i=0; i < readBytes; i++) 
      {
        freqCount[(0x000000ff & buffer[i])]++;
      }
      
      ArrayList list = new ArrayList();
      for (int i=0; i < 256; i++) {
        list.add(new Ranking(i, freqCount[i]));
      }
      Collections.sort(list);
      Iterator iter = list.iterator();
      _code = new String[256];
      int index = 0;
      _inverseMap = new HashMap();
      while (iter.hasNext()) 
      {
        Ranking rank = (Ranking) iter.next();
        _code[rank.getValue()]=map[index];
        _inverseMap.put(map[index], new Integer(rank.getValue()));
        index++;
      }
      
      int totalChars = 0;
      for (int i=0; i < readBytes; i++) 
      {
        String codeValue = _code[buffer[i] & 0xff];
        totalChars += codeValue.length();
        encodedString.append(_code[buffer[i] & 0xff]);
      }
      
    return ""+encodedString;
  }
  
  public String encodeHuffman(byte[] buffer, boolean std) 
  {
      StringBuffer encodedString = new StringBuffer();

      int readBytes = buffer.length;
      
      ArrayList list = new ArrayList();
      for (int i=0; i < 256; i++) 
      {
        list.add(new Ranking(i, i));
      }
      //Collections.sort(list);
      Iterator iter = list.iterator();
      _code = new String[256];
      int index = 0;
      _inverseMap = new HashMap();
      while (iter.hasNext()) 
      {
        Ranking rank = (Ranking) iter.next();
        _code[rank.getValue()]=map[index];
        _inverseMap.put(map[index], new Integer(rank.getValue()));
        index++;
      }
      
      int totalChars = 0;
      for (int i=0; i < readBytes; i++) {
        String codeValue = _code[buffer[i] & 0xff];
        totalChars += codeValue.length();
        encodedString.append(_code[buffer[i] & 0xff]);
      }
      
      return ""+encodedString;
  }
 
  public void buildInverseMap()
  {
      ArrayList list = new ArrayList();
      for (int i=0; i < 256; i++) 
      {
        list.add(new Ranking(i, i));
      }
      
      Iterator iter = list.iterator();
      _code = new String[256];
      int index = 0;
      _inverseMap = new HashMap();
      while (iter.hasNext()) 
      {
        Ranking rank = (Ranking) iter.next();
        _code[rank.getValue()]=map[index];
        _inverseMap.put(map[index], new Integer(rank.getValue()));
        index++;
      }  
  }
  
}
