/*
 * File.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.util;

import java.io.*;
import java.util.*;

/**
 *
 * @author  Lars Andersen
 */
public class RimfaxeFile 
{
  private String filename = "";
  int error = 0;
  int filelength = 0;
  
  String fs = System.getProperty("file.separator");
  
  /** 
   * Creates new file handler 
   */
  public RimfaxeFile() 
  {  
  }
  
  /** 
   * Creates new file handler 
   */
  public RimfaxeFile(String fname) 
  {
    filename = convert(fname); 
    //System.out.println("FILE -> "+filename);
  }
  
  public boolean delete()
  {
    java.io.File f = new java.io.File(filename);
    return f.delete();
  }
  
  public String convert(String in)
  {
     StringBuffer buf = new StringBuffer();
     StringTokenizer tkz = new StringTokenizer(in,"/\\",true);
     while (tkz.hasMoreTokens())
     {
       String tmp = tkz.nextToken();
       if (tmp.equalsIgnoreCase("/")) buf.append(fs);
       else if (tmp.equalsIgnoreCase("\\")) buf.append(fs);
       else buf.append(tmp);
     }
     return ""+buf;
  }
  
  public int getError() { return error; }
  
  public String read(InputStream is)
  {
    StringBuffer output = new StringBuffer();
    try 
    {
      byte[] buf = new byte[1000];
      int res=0;
      
      res = is.read(buf);
      while(res>0) 
      {
        String s = new String(buf);
        s = s.substring(0,res);
        output.append( s );
        res = is.read(buf);
      }
      is.close();
    }
    catch(IOException exc) 
    {
      error = 1;
    }
    
    return output.toString();
  }

  public String read()
  {
    StringBuffer output = new StringBuffer();
    try {
      FileInputStream stream = new FileInputStream(filename);
      byte[] buf = new byte[1000];
      int res=0;
      
      res = stream.read(buf);
      while(res>0) {
        String s = new String(buf);
        s = s.substring(0,res);
        output = output.append( s );
        res = stream.read(buf);
      }
      stream.close();
    }
    catch(IOException exc) 
    {
      error = 1;
      output.append("error reading file : "+filename);
    }
    
    return output.toString();
  }
  
  public byte[] readBytes()
  {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();  
    try 
    {
      FileInputStream stream = new FileInputStream(filename);
       
      byte[] buf = new byte[1000];
      int res=0;
      
      res = stream.read(buf);
      while(res>0) 
      {
        baos.write(buf,0,res);
        res = stream.read(buf);
      }
      stream.close();
    }
    catch(IOException exc) 
    {
      error = 1;
      com.rimfaxe.util.Log.log("error reading file : "+filename);
    }
    
    return baos.toByteArray();
  }
  
  public byte[] readBytes(InputStream stream)
  {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();  
    try 
    {
     
      byte[] buf = new byte[1000];
      int res=0;
      
      res = stream.read(buf);
      while(res>0) 
      {
        baos.write(buf,0,res);
        res = stream.read(buf);
      }
      stream.close();
    }
    catch(IOException exc) 
    {
      error = 1;
      
    }
    
    return baos.toByteArray();
  }
  
  public byte[] readBytesDeflated()
  {
    int l = 0;
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    java.util.zip.DeflaterOutputStream dos = new java.util.zip.DeflaterOutputStream(baos);
    try 
    {
      FileInputStream stream = new FileInputStream(filename);
       
      byte[] buf = new byte[1000];
      int res=0;
      
      res = stream.read(buf);
      while(res>0) 
      {
        l += res;
        dos.write(buf,0,res);
        
        res = stream.read(buf);
      }
      stream.close();
      dos.finish();
      filelength = l;
    }
    catch(IOException exc) 
    {
      error = 1;
      com.rimfaxe.util.Log.log("error reading file : "+filename);
    }
    
    return baos.toByteArray();
  }
  
  public byte[] deflateByteArray(byte[] inflatedarray)
  {
    int l = 0;
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    java.util.zip.DeflaterOutputStream dos = new java.util.zip.DeflaterOutputStream(baos);
    try 
    {
      ByteArrayInputStream stream = new ByteArrayInputStream(inflatedarray);
       
      byte[] buf = new byte[1000];
      int res=0;
      
      res = stream.read(buf);
      while(res>0) 
      {
        l += res;
        dos.write(buf,0,res);
        
        res = stream.read(buf);
      }
      stream.close();
      dos.finish();
      filelength = l;
    }
    catch(IOException exc) 
    {
      error = 1;
      
    }
    
    return baos.toByteArray();
  }  
  
  public int length() { return filelength; }
  
  public byte[] inflateByteArray(byte[] zip)
  {
    ByteArrayInputStream bais = new ByteArrayInputStream(zip);
    java.util.zip.InflaterInputStream stream = new java.util.zip.InflaterInputStream(bais);
    
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    
    try 
    {
      byte[] buf = new byte[1000];
      int res=0;
      
      res = stream.read(buf);
      while(res>0) 
      {
        baos.write(buf,0,res);
        
        res = stream.read(buf);
      }
      stream.close();
      
    }
    catch(IOException exc) 
    {
        
      error = 1;
    }
    
    return baos.toByteArray();
  }
  
  public void rewrite(String content)
  {
    int res = 0;
    try
    {
      FileOutputStream stream = new FileOutputStream(filename);
      DataOutputStream data   = new DataOutputStream(stream);
      
      data.writeBytes(content);
      
      data.close();
      stream.close();
    }
    catch (IOException exc) 
    {
      error = 2;  
    }
  }
  
  public void rewriteBytes(byte[] content)
  {
    int res = 0;
    try
    {
      FileOutputStream stream = new FileOutputStream(filename);
      stream.write(content);
      stream.close();
    }
    catch (IOException exc) 
    {
      error = 2;  
    }
  }
}
