/*
 * Template.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
package com.rimfaxe.util;

import java.util.*;

/** 
 *
 * @author  Lars A
 */
public class Template extends Object 
{
  String tag = "";
  String valuetoken = "";
  String templ = "";
  float rehash = (float) 0.90;
  Map replace = Collections.synchronizedMap(new HashMap(30, rehash)); 
  
  StringTokenizer tkz = null;
  boolean tkz_inside = false;

  /** 
   * Creates new template 
   */
  public Template(String str) 
  {
    this.templ = new String(str);
  }
  
  public Template(String str, String tag) 
  {
    this.tag = tag;
    this.templ = new String(str);
  }
  
  public void replace(String from, String to)
  {
    this.replace.put(from.toLowerCase(), to);
  }
   
  
  public String getNextValue(String token)
  {
    String val = null;
    String tmp = new String("");
    
    if (!token.equalsIgnoreCase(valuetoken)) 
    {
      valuetoken = token;
      tkz = new StringTokenizer(templ,"<=>",true);
      tkz_inside=false;
    }
    if (!tkz_inside)
    {
      if ( tag.equalsIgnoreCase("")) tkz_inside=true;
      else while ( tkz.hasMoreElements() && (!tmp.equalsIgnoreCase(tag)) ) tmp = tkz.nextToken();
      tkz_inside=true;
    }
    
    boolean found = false;
    
    while ( tkz.hasMoreElements() && (!found) )
    {
      tmp = tkz.nextToken();
      if ( tmp.equalsIgnoreCase(token) )
      {
        tkz.nextToken();
        val = tkz.nextToken();
        
        if (this.replace.containsKey(tmp+"="+val))
        {
          val = null;
          found = false;
        }
        else
          found = true;
        
      }
      if ( tmp.equalsIgnoreCase("/"+tag) ) 
      {
        tkz = new StringTokenizer(templ,"<=>",true);
        tkz_inside=false;
        return null;
      } 
    }
    return val;
  }
  
  public void reset()
  {
    float rehash = (float) 0.90;
    replace = Collections.synchronizedMap(new HashMap(30, rehash));
    tkz = new StringTokenizer(templ,"<=>",true); 
    tkz_inside=false;
  }
  
  public String simpleParser()
  {
    StringBuffer out = new StringBuffer();  
    StringTokenizer tok = new StringTokenizer(templ,"<>",true);
    String tmp = "";
    while (tok.hasMoreElements())
    {
      tmp = tok.nextToken();
      if ( replace.containsKey(tmp) )
      {
        out.deleteCharAt(out.length()-1);
        out.append( "" + this.replace.get(tmp) );    
        tok.nextToken();
      }
      else out.append(tmp);
      
    }
    return out.toString();
  }

  public String tagParser()
  {
    boolean inside     = false;
    boolean found      = false;

    StringBuffer out = new StringBuffer();  
    StringTokenizer tok = new StringTokenizer(templ,"<>",true);
    String tmp = "";
    while ( (tok.hasMoreElements()) && (!found) )
    {
      tmp = tok.nextToken();
    
      if ( tmp.equalsIgnoreCase("/"+tag) ) 
      {
        out.deleteCharAt(out.length()-1);
        inside=false;
        tok.nextToken();
        found = true;
      }
    
      if (inside)
      {
        if ( replace.containsKey(tmp) )
        {
          out.deleteCharAt(out.length()-1);
          out.append( "" + this.replace.get(tmp) );
          tok.nextToken();
        }
        else out.append(tmp);
      }
    
      if ( tmp.equalsIgnoreCase(tag) )
      {
        inside=true;
        tok.nextToken();
      }
    }
    return out.toString();
  }

  public String toString()
  {
    if (tag.equalsIgnoreCase(""))
      return simpleParser();
    else
      return tagParser();
  }
   
}