/*
 * Validate.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.util;

import java.net.*;
import java.util.*;
import java.io.*;

/**
 *
 * @author  Lars Andersen
 */
public class Validate 
{
    
    /** Creates a new instance of Validate */
    public Validate() 
    {}
    
    public static boolean validateStringToInteger(String i)
    {
       boolean res = true;
       try
       {
          Integer t = new Integer(i);
       }
       catch (NumberFormatException nfe) 
       {
         res = false;
       }
       return res;
    }
     
    public static boolean validateStringToInetAddress(String i)
    {
       boolean res = true;
       try
       {
          InetAddress ia = InetAddress.getByName(i); 
       }
       catch (UnknownHostException uhe) 
       {
         res = false;
       }
       catch (Exception e) 
       {
         res = false;
       }
       
       if (i.equalsIgnoreCase("")) res = false;
       
       return res;
    }
    
    public static boolean validateStringToIP(String ip)
    {
       boolean res = true;
       try
       {
          byte[] addr = new byte[4];
          StringTokenizer tkz = new StringTokenizer(ip,".",false);
          Integer i = new Integer(tkz.nextToken());
          addr[0] = i.byteValue();
          i = new Integer(tkz.nextToken());
          addr[1] = i.byteValue();
          i = new Integer(tkz.nextToken());
          addr[2] = i.byteValue();
          i = new Integer(tkz.nextToken());
          addr[3] = i.byteValue(); 
          InetAddress ia = InetAddress.getByName(ip);
       }
       catch (UnknownHostException uhe) 
       {
         res = false;
       }
       catch (Exception e) 
       {
         res = false;
       }
       
       return res;
    }
    
    public static  boolean validateAdmin(String in)
    {
      if (in.length()>=4)
      {
        //System.out.println("Length of ["+in+"] is more than 4 letters");  
        return true;
      }
      else
      {
        //System.out.println("Length of ["+in+"] is less than 4 letters");  
        return false;
      }
    }
    
    public static boolean validateStringToInterface(String i)
    {
       if (i.equalsIgnoreCase("eth0")) return true;
       if (i.equalsIgnoreCase("eth1")) return true;
       if (i.equalsIgnoreCase("eth2")) return true;
       if (i.equalsIgnoreCase("eth3")) return true;
       if (i.equalsIgnoreCase("eth4")) return true;
       if (i.equalsIgnoreCase("eth5")) return true;
       if (i.equalsIgnoreCase("eth6")) return true;
       if (i.equalsIgnoreCase("eth7")) return true;
       if (i.equalsIgnoreCase("eth8")) return true;
       if (i.equalsIgnoreCase("eth9")) return true;
       return false; 
    }
    
    public static boolean validateStringToFile(String i)
    {
       RimfaxeFile file = new RimfaxeFile(i);
       String c = file.read();
       if (c.startsWith("error reading")) return false;
     
       return true;
    }
     
}
