/*
 * Configuration.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver;

import java.util.*;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


import com.rimfaxe.util.*;

/**
 *
 * @author  Lars Andersen
 */
public class Configuration 
{
  Edition edition = new Edition();
  
  boolean debug = true;
  boolean active = true;
  
  String webapp_mode = "rws";
  
  RimfaxeVector daemons = new RimfaxeVector();
  
  Vector virtuallist = new Vector();
  
  String admin;
  String password;
  String RWShome = "";
  
  String CryptoProvider = "RIMFAXE";
  
  RimfaxeVector sharedlibs = new RimfaxeVector();
  
  //com.rimfaxe.webserver.daemon.ServerHandlerManager serverhandlermanager = null;
  
  String dhfile ="dh1024.pem";
  String certfile = "req.pem"; 
  String sslpassphrase = "lamatop";
  
  com.rimfaxe.webserver.webapp.Web webapp_default = new com.rimfaxe.webserver.webapp.Web();
  
  String gcj_compiler = "/usr/local/bin/gcj";
  
  int cache_size = 32000;
  
  String timezone = "Europe/Copenhagen";
  
  String java_encoding = "ISO-8859-1";
  
  /** Creates a new instance of Configuration */
  public Configuration() 
  {
    com.rimfaxe.util.RimfaxeFile file = new com.rimfaxe.util.RimfaxeFile("config/rimfaxe-webserver.xml");
    String xml = file.read();
    
    com.rimfaxe.xml.ParserInterface parser = com.rimfaxe.xml.ParserFactory.getInstance().checkoutParser();
   
    parser.parse(xml,false);
    Document document = parser.getDocument(); 
   
    traverse(document); 
    
    
  }
   
  
  /** Creates a new instance of Configuration */
  public Configuration(String xml, boolean validate) 
  {
    com.rimfaxe.xml.ParserInterface parser = com.rimfaxe.xml.ParserFactory.getInstance().checkoutParser();
    parser.parse(xml,false);
    Document document = parser.getDocument(); 
    
    traverse(document); 
  }
 
  
  public void initialize()
  {
    com.rimfaxe.webserver.compiler.GCJcompiler gcj = new  com.rimfaxe.webserver.compiler.GCJcompiler(); 
    
    // Load shared libs
    loadShared();
    
    Enumeration enum = virtuallist.elements();
    while (enum.hasMoreElements())
    {
      VirtualHost vh = (VirtualHost) enum.nextElement();
      
      vh.initialize();
    }
    
    
  }
   
  public void loadShared()
  {
     //System.out.println("Load shared"); 
     java.io.File root = null;
     
     try
     {
       //System.out.println("Load shared 1");   
       root = new java.io.File( this.getHome() + "/shared");
       
       if (!root.exists()) root.mkdir();
     }
     catch (Exception e) {}
     
     if (root==null) return;
     
     String[] dirlist = root.list();
     
     //System.out.println("Load shared 2"); 
     
     if (dirlist==null) return;
     
     //System.out.println("Load shared 3");   
     
     for (int i=0; i<dirlist.length; i++)
     {
       String f = dirlist[i];
       
       java.io.File temp = new java.io.File(this.getHome() + "/shared"+"/"+f);
       
       if (temp.isDirectory())
       {
         //skip  
       }
       else
       {
         if (  (f.endsWith(".so"))  && (f.startsWith("lib")) )
         {
           try
           {
             
             
             String libname = f.substring(3,f.length()-3);
             System.out.println("Shared lib ["+libname+"]");
             sharedlibs.addElement(libname);
           }
           catch (Exception e) {}
         }
       }
     }
  }
  
  public Enumeration getSharedLibs() { return this.sharedlibs.elements(); }
  
  public com.rimfaxe.webserver.webapp.Web getWebApp() { return webapp_default; }
  public String getCertfile() { return certfile; }
  public String getDHfile() {  return dhfile; }
  public String getSSLpassphrase() {  return sslpassphrase; }
  public String getCryptoProvider() { return CryptoProvider; }
  
  public String getWebappMode() { return webapp_mode; }
  
  public String getAdminIP()
  {
    return "127.0.0.1";  
  }
  
  public Integer getAdminPort() 
  {
    return new Integer(8111);  
  }
  
  public void save()
  {
    com.rimfaxe.util.RimfaxeFile file = new  com.rimfaxe.util.RimfaxeFile("config/rimfaxe-webserver.xml");
    String xml = toXML();
    file.rewrite(xml);   
  }
 
  public boolean isTrial() { return edition.isTrial(); }
  public long getRunningTime() { return edition.getRunningTime(); }
  
  public void shutdown()
  {   
    //serverhandlermanager.shutdown();
    System.out.println("Shutdown finished"); 
    System.exit(0);
  }
  
    
  public VirtualHost getVirtualHost(String id)
  {
    Enumeration enum = virtuallist.elements();
    while (enum.hasMoreElements())
    {
      VirtualHost vhost = (VirtualHost) enum.nextElement();
      if (vhost.acceptHost(id)) return vhost;
    }
    return null;
  }
  
  
  
  public String getSerial() { return edition.getSerial(); }
  public String getEdition() { return edition.getEdition(); }
  public String getVersion() { return edition.getVersion(); }
  public String getBanner() { return edition.getBanner(); }
  public String getOS() { return edition.getOS(); }
  
  public String getAdmin() { return admin; }
  public String getPassword() { return password; }
  public String getHome() { return RWShome; }
  
  public void setHome(String dir)
  {
    RWShome = dir;
  }
  
  public void setAdmin(String admin)
  {
   this.admin = admin;
  }
  
  public void setPassword(String password)
  {
   this.password = password;
  }
  
  public void setCertfile(String c)
  {
   this.certfile = c;
  }
  
  public void setDHfile(String dh)
  {
   this.dhfile = dh;
  }
  
  public void setSSLpassphrase(String pass)
  {
   this.sslpassphrase = pass;
  }
  
  
  public Enumeration getVHostList() { return virtuallist.elements(); }
  
  public Enumeration getVHostPortList() 
  {
     RimfaxeVector ports = daemons; 
     Enumeration enum = virtuallist.elements(); 
     while (enum.hasMoreElements())
     {
        VirtualHost vh = (VirtualHost) enum.nextElement();
        Enumeration enum2 = vh.getConnectors().elements();
        while (enum2.hasMoreElements())
        {
          Connector con = (Connector) enum2.nextElement();
          ServerPort sp = new ServerPort(con.getIP(), new Integer(con.getPort()), con.getProtocol());
          if (!ports.contains(sp))
          {
            ports.addElement(sp);
          }
        }
        
     }
     daemons = ports;
     return ports.elements();
  }
  
  public RimfaxeVector getDaemons() { return daemons; }
  
  
  public String getGCJ() { return this.gcj_compiler; }
  public String getJavaEncoding() { return this.java_encoding; } 
  
  public int getCacheSize() { return this.cache_size; }
  
 
  
  private String traverse(Node node)
  {
    
    StringBuffer str = new StringBuffer();
    
    if (node == null) 
    {
      return ""; 
    }
    int type = node.getNodeType();
    switch (type) 
    {
      
      case Node.DOCUMENT_NODE: 
      { 
        traverse(((Document)node).getDocumentElement());
        break;
      }
      
      case Node.ELEMENT_NODE: 
      {  
        if (node.getNodeName().equalsIgnoreCase("virtual"))
        {  
          //com.rimfaxe.util.Log.log("Virtual server found");    
          VirtualHost vh = new VirtualHost(node);
          
          
          virtuallist.addElement(vh);
          
          
        }
        else if (node.getNodeName().equalsIgnoreCase("admin"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              admin = val.trim().toLowerCase();
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("password"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              password = val.trim().toLowerCase();
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("gcj"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              gcj_compiler = val.trim();
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("JavaEncoding"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              java_encoding = val.trim();
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("timezone"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              timezone = val.trim();
              try
              {
                com.rimfaxe.util.Calendar.setDefaultTimeZone( TimeZone.getTimeZone(timezone)  );  
              }
              catch (Exception e)
              {
                System.err.println("Invalid timezone : "+timezone);   
              }
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("webapp-mode"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              
              if (val.trim().equalsIgnoreCase("rws")) webapp_mode = "rws";
              if (val.trim().equalsIgnoreCase("classic")) webapp_mode = "classic";
              
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("cache-size"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i));
              try
              {
                Integer csize = new Integer(val.trim());
                cache_size = csize.intValue();
              }
              catch (Exception e) {}
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("sslpassphrase"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              sslpassphrase = val.trim().toLowerCase();
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("certfile"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              certfile = val.trim().toLowerCase();
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("dhfile"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              dhfile = val.trim().toLowerCase();
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("home"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              RWShome = val.trim();
            }
          }    
        }
        else
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
            }
          }      
        }
        break;
      }
      
      case Node.TEXT_NODE: 
      {
        str.append(node.getNodeValue());
        break;
      }
    }
    return str.toString();
  }
  
  public String toXML()
  {
    return toXML(false);  
  }
  
  public String snapshot()
  {
    return toXML(true);  
  }
  
  public String toXML(boolean snapshot)
  {
    StringBuffer buf = new StringBuffer();   
      
    buf.append("<?xml version='1.0'?>\n\n<document>\n\n");
    
    buf.append("  <admin> "+admin+ " </admin>\n");
    buf.append("  <password> "+password+" </password>\n\n");
    
    buf.append("  <home> "+RWShome+" </home>\n\n");
    buf.append("  <gcj> "+gcj_compiler+" </gcj>\n\n");
    buf.append("  <cache-size> "+cache_size+" </cache-size>\n\n");
    buf.append("  <webapp-mode> "+webapp_mode+" </webapp-mode>\n\n");
    
    Enumeration enum = virtuallist.elements();
    while (enum.hasMoreElements())
    {
      VirtualHost vh = (VirtualHost) enum.nextElement();  
      buf.append(""+vh.toXML(snapshot));
    }
    
    buf.append("\n\n</document>");
    return ""+buf;
  }
  
  public String format(String in, int w)
  {
    StringBuffer buf = new StringBuffer();   
    
    int c = 0;
    
    while ( (c+w)<=in.length() )
    {
      String tmp = in.substring(c,c+w);
      c += w;
      buf.append(tmp+"\n");
    }
    
    if (c<in.length()) buf.append( in.substring(c,in.length()) );
    
    return ""+buf;
  }
}
