/*
 * Connector.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver;

import java.util.*;
import com.rimfaxe.util.*;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 * @author  Lars Andersen
 */
public class Connector 
{
  String ip = "127.0.0.1";
  String hostname = "localhost";
  int port = 8081;
  
  String server = "httpd";
  
  String protocol = "http";
  
  RimfaxeVector aliases = new RimfaxeVector();
  
  /** Creates a new instance of Connector */
  public Connector(Node node) 
  {
    server = node.getAttributes().getNamedItem("server").getNodeValue().trim();  
    ip = node.getAttributes().getNamedItem("ip").getNodeValue().trim();
    hostname = node.getAttributes().getNamedItem("hostname").getNodeValue().trim();
    Integer port_i = new Integer(node.getAttributes().getNamedItem("port").getNodeValue());
    port = port_i.intValue();
    
    protocol = node.getAttributes().getNamedItem("protocol").getNodeValue().trim();
   
    aliases.addElement(hostname+":"+port);
    
    traverse(node);
  }

  public boolean acceptHost(String str)
  {
    
    if (aliases.contains(str)) return true;
    return false;
  }
  
  public String getServer() { return server; }
  
  public String getProtocol() { return protocol; }
  public String getIP() { return ip; }
  public String getHostname() { return hostname; }
  
  public int getPort() { return port; }
  
  public boolean acceptAlias(String hname)
  {
      
    return true;
  }
  
  private String traverse(Node node)
  {
    
    StringBuffer str = new StringBuffer();
    
    if (node == null) 
    {
      return ""; 
    }
    int type = node.getNodeType();
    switch (type) 
    {
      
      case Node.DOCUMENT_NODE: 
      { 
        traverse(((Document)node).getDocumentElement());
        break;
      }
      
      case Node.ELEMENT_NODE: 
      {  
        if (node.getNodeName().equalsIgnoreCase("alias"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              aliases.addElement( val.trim() );
            }
          }    
        }
        else
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
            }
          }      
        }
        break;
      }
      
      case Node.TEXT_NODE: 
      {
        str.append(node.getNodeValue());
        break;
      }
    }
    return str.toString();
  }
  
  public String toXML()
  {
    StringBuffer buf = new StringBuffer();   
      
    buf.append("    <connector hostname=\""+hostname+"\" "+
               "ip=\""+ip+"\" port=\""+port+"\" server=\""+server+"\" "+
               "protocol=\""+protocol+"\">\n");
       
    Enumeration enum = aliases.elements();
    while (enum.hasMoreElements())
    {
      String a = (String) enum.nextElement();
      buf.append("      <alias> "+a+" </alias>\n");
    }
    
    
    buf.append("    </connector>\n");
    
    return ""+buf;
  }  
}
