/*
 * ErrorLog.java
 *
 *
 * Copyright (c) 2004 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver;

import java.io.*;
import java.util.*;
import java.text.*;

/**
 *
 * @author  lars
 */
public class ErrorLog 
{
  private static BufferedWriter bw = null;
  private static FileWriter fw;
  
  private static String logFileName = "log/error.log";
  private static long logFileSize = 100*1024;
  
  private final static int numberOfEntries = 100;
  
  private static String[] lastEntries = new String[numberOfEntries];
  private static int index = 0;
  private static boolean firstTime = true;
  private static int firstEntry = 0;
  
  /** Creates a new instance of ErrorLog */
  public ErrorLog() 
  {
  }
    
  public static void setLogFile(String name)
  {
    logFileName = name;
  }
  
  private static void newLogFile()
  {
    try
    {
      fw = new FileWriter(logFileName, true); // append to old file
      bw = new BufferedWriter(fw);
    }
    catch(Exception e)
    {
      String str = new String(DateFormat.getDateTimeInstance().format(new Date()) + "Exception occured in Log.write: " + e);
      try
      {
        bw.write(str, 0, str.length());
        bw.newLine();
      }
      catch(Exception ex)
      {
      }
    }
  }
  
  private static void checkFile()
  {
    // Check if the stream is open, if not create a new stream.
    if(bw == null)
      newLogFile();

    java.io.File logFile = new java.io.File(logFileName);

    // If the log file is greater than logFileSize, rename it to "logfile.old"
    // and create a new log file.
    try
    {
      if(logFile.length() > logFileSize)
      {
          
        java.io.File oldLogFile = new java.io.File(logFileName+"_"+DateFormat.getDateTimeInstance().format(new Date())+".old");

        if(oldLogFile.exists())
          oldLogFile.delete();
        bw.close(); // closes the current stream
        bw = null;
        if(logFile.renameTo(oldLogFile) != true)
        {
          String str = new String(DateFormat.getDateTimeInstance().format(new Date()) + "Error, can't rename to "+oldLogFile);
          System.err.println(str);
        }
        newLogFile();
      }
    }
    catch(Exception e)
    {
      //e.printStackTrace();
    }
  }
    
    
  private static synchronized void write(String logString)
  {
    try
    {
      checkFile();
      bw.write(logString, 0, logString.length());
      bw.newLine();
      bw.flush();
      lastEntries[index] = logString;
      if(!firstTime) 
      {
        if(index==numberOfEntries - 1)
          firstEntry = 0;
        else
          firstEntry = index+1;
      }
      
      index++;
      if (index>=numberOfEntries) 
      {
        firstTime = false;
        index=0;
      }
        
    }
    catch(IOException e)
    {
    }
    catch(Exception e)
    {
    }
  }


  private static String getMiliSec()
  {
    String t = Long.toString(new Date().getTime());
    return ("."+t.substring((t.length() -3), t.length()));
  }

  
  public static void log(String class_name, int linenumber, String error, String description)
  {
    com.rimfaxe.util.Calendar cal = new com.rimfaxe.util.Calendar();  
    write("\n"+
          "----------------------------------------------------------------------------------\n"+
          ""+cal.getTimestamp()+getMiliSec()+"\n"+
          ""+class_name+"["+linenumber+"]"+"\n"+
          ""+error+"\n"+
          ""+description+"\n"+
          "==================================================================================\n\n");
  }
  
  
  
  
}
