/*
 * ObjectStore.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver;

import java.util.*;


/**
 *
 * @author  Lars Andersen
 */
public class ObjectStore 
{
  Configuration conf = new Configuration();
 
  /** Handle ObjectStore as a singleton */
  public static ObjectStore instance;
 
  
  public synchronized static ObjectStore getInstance() 
  {
    if (instance==null)
    {
      
      instance = new ObjectStore();
     
    }
    return instance;
  }  
  
  public synchronized void createPool()
  {  
    for (int i=0;i<10;i++)
    {
      handlers.addElement(createHandler());
    }
  }
  
  public Handler createHandler()
  {
    //System.out.println("\n\nCreating new Handler\n\n");  
    Handler h = new Handler(conf);
    return h;
  }
  
  
  Vector handlers = null;
  

  /** Creates a new instance of ObjectStore */
  public ObjectStore() 
  {
     
    handlers = new Vector();
  }
  
  public synchronized Handler getHandler()
  {
    if (handlers.size()==0)  
    {
      Handler h = createHandler();
      return h;
    }
    else
    {
      Handler h = (Handler) handlers.remove(0);
      if (h.getAge()>(5*60*60*1000))
         h = createHandler(); 
      else
         h.reset();
      return h;   
    }    
  }
  
  public synchronized void putHandler(Handler h)
  {
     handlers.addElement(h);
  }
  
  public static Handler checkoutHandler()
  {
    return getInstance().getHandler();  
  }
  
  public static void checkinHandler(Handler h)
  {
    getInstance().putHandler(h);  
  }
  
  
  public static Configuration getConfiguration()
  {
    return getInstance().conf;  
  }
  
  public static void setConfiguration(Configuration conf)
  {
    getInstance().conf = conf;  
  }
  
  public int getNumberOfHandlers()
  {
    return handlers.size();  
  }
  
  public static int getSize()
  {
    return getInstance().getNumberOfHandlers();  
  }
  
 

  
}
