/*
 * ServerPort.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver;


import com.rimfaxe.webserver.stat.DaemonStat;



/**
 *
 * @author  Lars Andersen
 */
public class ServerPort 
{
  String ip = "";
  Integer port = new Integer(80);
  String protocol = "http";
  
  com.rimfaxe.webserver.seda.Server daemon = null;
  
  DaemonStat stat = new DaemonStat();
  
  /** Creates a new instance of ServerPort */
  public ServerPort(String ip, Integer port) 
  {
      this.ip = ip;
      this.port = port;
      
  }
  
  /** Creates a new instance of ServerPort */
  public ServerPort(String ip, Integer port, String protocol) 
  {
      this.ip = ip;
      this.port = port;
      this.protocol = protocol;
      
  }
  
  public String getIP() { return ip; }
  public Integer getPort() { return port; }
  public String getProtocol() { return protocol; }
  
  
  public DaemonStat getStat() { return stat; }
  
  
  public void addDaemon(com.rimfaxe.webserver.seda.Server daemon)
  {
    this.daemon = daemon;  
  }
  
  public com.rimfaxe.webserver.seda.Server getDaemon()
  {
    return daemon;
  }
  
  public String getDesc() { return ip+":"+port; }
  public String toString() { return ""+port; }
  
  public com.rimfaxe.xml.datamodel.Container getData()
  {
    com.rimfaxe.xml.datamodel.Container data = new com.rimfaxe.xml.datamodel.Container();
    data.setName("serverport");
    
    data.addLiteral("protocol",protocol);
    data.addLiteral("port",""+port);
    
    //System.out.println("adding serverport stat");
    data.addContainer("daemonstat",stat.getData());
    
    return data;
  }
}
