/*
 * VirtualHost.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver;

import java.util.*;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


import com.rimfaxe.util.*;

/**
 *
 * @author  Lars Andersen
 */
public class VirtualHost 
{
  String name = "";
  
  boolean admin = false;
  boolean isdefault = false;
  boolean debug = false;
  boolean trace = false;
  
  RimfaxeVector connectors = new RimfaxeVector();
  RimfaxeVector webcontexts = new RimfaxeVector();
  
  com.rimfaxe.webserver.servletapi.session.SessionStore sessionstore;
  
  Configuration conf = null;
  
  /** Creates a new instance of VirtualHost */
  public VirtualHost(Node node) 
  {  
    name = node.getAttributes().getNamedItem("name").getNodeValue().trim();   
    traverse(node);  
     
    sessionstore = new com.rimfaxe.webserver.servletapi.session.SessionStore(this);
    
    
  }
    
  public com.rimfaxe.webserver.servletapi.RimfaxeServletContext getServletContext(String str)
  {
    com.rimfaxe.webserver.servletapi.RimfaxeServletContext best = null;  
    int longestmatch = -1;  
    Enumeration enum = webcontexts.elements();
    while (enum.hasMoreElements())
    {
      WebContext ctx = (WebContext) enum.nextElement();
      int m = ctx.matchURI(str);
      if (m>longestmatch) 
      {
        best = ctx.getServletContext();
        longestmatch = m;
      }
    }
    return best;
  }
  
  public void initialize()
  {
    com.rimfaxe.webserver.compiler.GCJcompiler gcj = new  com.rimfaxe.webserver.compiler.GCJcompiler(); 
    Enumeration enum = webcontexts.elements();
    while (enum.hasMoreElements())
    {
      WebContext webcontext = (WebContext) enum.nextElement();
      
      gcj.compileWebApp(this,webcontext);
      
      webcontext.initialize();
    }
  }
  
  
  public boolean acceptHost(String str)
  {
    Enumeration enum = connectors.elements();
    while (enum.hasMoreElements())
    {
      Connector c = (Connector) enum.nextElement();
      if (c.acceptHost(str)) return true;
    }  
    
    return false;
  }
  
  public RimfaxeVector getConnectors() { return connectors; }
  
  public com.rimfaxe.webserver.servletapi.session.SessionStore getSessionStore()
  {
    return sessionstore;   
  }
  
  public String getName() { return name; }
  
  public boolean getDebug() { return debug; }
  
  public WebContext getWebContext(String str)
  {
    WebContext best = null;  
    int longestmatch = -1;  
    Enumeration enum = webcontexts.elements();
    while (enum.hasMoreElements())
    {
      WebContext ctx = (WebContext) enum.nextElement();
      int m = ctx.matchURI(str);
      if (m>longestmatch) 
      {
        best = ctx;
        longestmatch = m;
      }
    }
    return best;  
  }
   
  public boolean allowAdmin() { return this.admin; }
  public boolean isDefault() { return this.isdefault; }
  
  private String traverse(Node node)
  {
    
    StringBuffer str = new StringBuffer();
    
    if (node == null) 
    {
      return ""; 
    }
    int type = node.getNodeType();
    switch (type) 
    {
      
      case Node.DOCUMENT_NODE: 
      { 
        traverse(((Document)node).getDocumentElement());
        break;
      }
      
      case Node.ELEMENT_NODE: 
      {  
        //System.out.println("node = "+node.getNodeName());  
        
        if (node.getNodeName().equalsIgnoreCase("connector"))
        {
          connectors.addElement( new Connector(node));
        }
        else if (node.getNodeName().equalsIgnoreCase("webcontext"))
        {
          webcontexts.addElement( new WebContext(this,node));
        }
        else if (node.getNodeName().equalsIgnoreCase("debug"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              if ( val.trim().equalsIgnoreCase("true") )
                debug = true;
              else
                debug = false;
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("trace"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              if ( val.trim().equalsIgnoreCase("true") )
                trace = true;
              else
                trace = false;
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("admin"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              if ( val.trim().equalsIgnoreCase("true") )
                admin = true;
              else
                admin = false;  
            }
          }    
        }
        else
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
            }
          }      
        }
        break;
      }
      
      case Node.TEXT_NODE: 
      {
        //if (node instanceof TextImpl) 
        //{ 
          str.append(node.getNodeValue());
        //} 
        break;
      }
    }
    return str.toString();
  }
  
  public String toXML()
  {
    return toXML(false);  
  }
  
  public String toXML(boolean snapshot)
  {
    StringBuffer buf = new StringBuffer();   
      
    buf.append("  <virtual>\n\n");
       
    Enumeration enum = connectors.elements();
    while (enum.hasMoreElements())
    {
      Connector c = (Connector) enum.nextElement();
      buf.append( c.toXML() );
    }
    
    buf.append("\n");
    buf.append("    <default> "+isdefault+ " </default>\n");
    buf.append("    <admin> "+admin+" </admin>\n\n");
    
    buf.append("    <debug> "+debug+ "</debug>\n");
    buf.append("    <trace> "+trace+" </trace>\n\n");
    
    enum = webcontexts.elements();
    while (enum.hasMoreElements())
    {
      WebContext c = (WebContext) enum.nextElement();
      buf.append( c.toXML() );
    }
    
    buf.append("  </virtual>\n\n");
  
    
    return ""+buf;
  }  
  
}
