/*
 * GCJcompiler.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.compiler;

import java.util.*;
import com.rimfaxe.webserver.VirtualHost;
import com.rimfaxe.webserver.WebContext;

/**
 *
 * @author Lars Andersen
 */
public class GCJcompiler 
{
  com.rimfaxe.webserver.Configuration configuration = com.rimfaxe.webserver.ObjectStore.getConfiguration(); 
  
  /** Creates a new instance of GCJcompiler */
  public GCJcompiler() 
  {
  }
  
  public void checkDirs(String workdir)
  {
    
    java.io.File dir = new java.io.File(workdir);
    if (!dir.exists()) dir.mkdir();
    
    
    dir = new java.io.File(workdir+"/jars");
    if (!dir.exists()) dir.mkdir();
    
    dir = new java.io.File(workdir+"/jsp");
    if (!dir.exists()) dir.mkdir();
    
    dir = new java.io.File(workdir+"/jsplib");
    if (!dir.exists()) dir.mkdir();
    
    dir = new java.io.File(workdir+"/lib");
    if (!dir.exists()) dir.mkdir();
    
    
  }
  
  public void compileWebApp(VirtualHost vh, WebContext webcontext)
  {
    boolean hasEXT = false;
    
    // compile extension lib  
     
    String webroot = webcontext.getRoot();  
    
      
    System.out.print("Compile libservlets.so in web context ["+webcontext.getName()+"]  ");  
     
    Library libservlets = new Library();
    libservlets.setOutputFile("libservlets.so");
    libservlets.setOutputDir(webroot+"/WEB-INF/rws/lib");
    
    checkDirs(webroot+"/WEB-INF/rws/");
    
    Vector inputfiles = new Vector();  
    Vector resourcefiles = new Vector(); 
    
    if (configuration.getWebappMode().equalsIgnoreCase("classic"))
    {
      
      libservlets.addCompileOption("-I"+webroot+"/WEB-INF/classes"); 
      libservlets.setInputDir(webroot+"/WEB-INF/classes");
      libservlets.setSource(libservlets.CLASS_SOURCE);
        
      findFiles(webroot+"/WEB-INF/classes","class",inputfiles);
      libservlets.setInputFiles(inputfiles);
    
      findResources(webroot+"/WEB-INF/classes",resourcefiles);
      libservlets.setResourceFiles(resourcefiles);
    }
    else
    {
      libservlets.addCompileOption("-I"+webroot+"/WEB-INF/rws/src");
      libservlets.setInputDir(webroot+"/WEB-INF/rws/src");
      libservlets.setSource(libservlets.JAVA_SOURCE);
         
      findFiles(webroot+"/WEB-INF/rws/src","java",inputfiles);
      libservlets.setInputFiles(inputfiles);
     
      findResources(webroot+"/WEB-INF/rws/src",resourcefiles);
      libservlets.setResourceFiles(resourcefiles);
    }
    
    
    
    
    if (hasEXT)
    {
      libservlets.addLinkOption("-lext -L"+webroot+"/WEB-INF/rws/lib");   
    }
    
    
    if (inputfiles.size()>0)
    {
      try
      {
          
        libservlets.compile();
      }
      catch (Exception e)
      {
         System.out.println(""+e);   
      }
    }
    System.out.println("");
    System.out.println("Compilation done");
    
  }
  
  public void findFiles(String rootdir, String ext, Vector vec)
  {
      
     java.io.File root = null;
     
     try
     {
       root = new java.io.File(rootdir);
     }
     catch (Exception e) {}
     
     if (root==null) return;
     
     String[] dirlist = root.list();
     
     if (dirlist==null) return;
     
     for (int i=0; i<dirlist.length; i++)
     {
       String f = dirlist[i];
       String name = f.toLowerCase();
       
       
       java.io.File temp = new java.io.File(rootdir+"/"+f);
       if (temp.isDirectory())
       {
         findFiles(rootdir+"/"+f, ext,  vec);  
       }
       else
       {
         if (name.endsWith("."+ext))
         {
           
           vec.addElement(rootdir+"/"+f);  
         }
       }
     }
  }
  
  public void findResources(String rootdir, Vector vec)
  {
     
     java.io.File root = null;
     
     try
     {
       root = new java.io.File(rootdir);
     }
     catch (Exception e) {}
     
     if (root==null) return;
     
     String[] dirlist = root.list();
     
     if (dirlist==null) return;
     
     for (int i=0; i<dirlist.length; i++)
     {
       String f = dirlist[i];
       String name = f.toLowerCase();
       
       java.io.File temp = new java.io.File(rootdir+"/"+f);
       if (temp.isDirectory())
       {
         findResources(rootdir+"/"+f,   vec);  
       }
       else
       {
         if (  (!name.endsWith(".java")) &&  (!name.endsWith(".class"))  &&  (!name.endsWith(".o")) &&  (!name.endsWith("~")) )
         {
           vec.addElement(rootdir+"/"+f);  
         }
       }
     }
  }
}
