/*
 * Library.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.compiler;

import java.util.*;

/**
 *
 * @author Lars Andersen
 */
public class Library 
{
  public final int JAR_SOURCE = 0;
  public final int CLASS_SOURCE = 1;
  public final int JAVA_SOURCE = 2;
  public final int OBJECT_SOURCE = 3;
  public final int MIXED_SOURCE = 4;
  
  int source = 0;
  
  com.rimfaxe.webserver.Configuration configuration = com.rimfaxe.webserver.ObjectStore.getConfiguration(); 
  
  String RWSlibrarydir = configuration.getHome()+"/lib";
  String RWSshareddir = configuration.getHome()+"/shared";
  String RWSextdir = configuration.getHome()+"/source";
  
  Vector inputfiles = new Vector();
  Vector resourcefiles = new Vector();
  
  Vector compileoptions = new Vector();
  
  Vector linkoptions = new Vector();
  
  String inputdir = "/";
  String outputdir = "/";
  String outputfile = "libext.so";
  
  
  int optimizations = 0;
  int opt_G         = 1;
  
  String compiler = configuration.getGCJ();
  
  /** Creates a new instance of Library */
  public Library() 
  {  
  }
  
  public void addCompileOption(String opt) { compileoptions.addElement(opt); }
  public void addLinkOption(String opt) { linkoptions.addElement(opt); }
  
  public void setOutputFile(String val)
  {
    outputfile = val;
  }
  
  public void setOutputDir(String val)
  {
    outputdir = val;
  }
  
  public void setInputDir(String val)
  {
    inputdir = val;
  }
  
  public void setInputFiles(Vector val)
  { 
    this.inputfiles = val;
  }
  
  public void setResourceFiles(Vector val)
  { 
    this.resourcefiles = val;
  }
  
  public void setSource(int i) { source = i; }
  
  /**
   * The jars is compiled to object files, and then linked into a dynamic so file.
   * Link against RWS shared lib.
   */
  public void compile()
      throws NativeCompilationException, NativeLinkException
  {
    
    linkoptions.addElement("-shared");  
    linkoptions.addElement("-L"+RWSlibrarydir);  
    linkoptions.addElement("-L"+RWSshareddir);
    
    linkoptions.addElement("-lrws");
   
    Enumeration enum = configuration.getSharedLibs();
    while (enum.hasMoreElements())
    {
       linkoptions.addElement("-l"+enum.nextElement());  
    }
    //compileoptions.addElement("-fno-assume-compiled");
    
    compileoptions.addElement("--encoding="+com.rimfaxe.webserver.ObjectStore.getConfiguration().getJavaEncoding());
    compileoptions.addElement("-O"+optimizations);
    compileoptions.addElement("-I"+RWSextdir);
    compileoptions.addElement("-c");
    compileoptions.addElement("-g"+opt_G);
    
   
    if (source==this.JAR_SOURCE) 
    {
      compileJARS();  
    }
    else if (source==this.JAVA_SOURCE) 
    {
      compileJAVA();  
    }
    else if (source==this.CLASS_SOURCE) 
    {
      compileJAVA();  
    }
    
  }
  
  private void compileJARS()
      throws NativeCompilationException, NativeLinkException
  {
    // compile each of the object files
    Enumeration enum = inputfiles.elements();
    while (enum.hasMoreElements())
    {
       String sourcefile = ""+enum.nextElement();  
       
       StringBuffer buf = new StringBuffer();
       buf.append(compiler+" ");
       appendCompileOptions(buf);
       appendClasspath(buf,inputfiles);
       
       buf.append(""+sourcefile+" ");
       buf.append("-o "+sourcefile+".o");
       
       executeCommand(""+buf,"compile");
       System.out.println("Library->compile  {"+buf+"}");
    }   
    
    // Link phase
    
    StringBuffer buf = new StringBuffer();
    buf.append(compiler+" ");
    appendLinkOptions(buf);
    buf.append("-o "+outputdir+"/"+outputfile+" ");
    
    enum = inputfiles.elements();
    while (enum.hasMoreElements())
    {
       String objfile = ""+enum.nextElement()+".o";  
       buf.append(objfile);
    }   
    executeCommand(""+buf,"link");
    //System.out.println("Library->link  {"+buf+"}");
    
  }
  
  private void compileJAVA()
      throws NativeCompilationException, NativeLinkException
  {
    // compile each of the object files
        
    Enumeration enum = inputfiles.elements();
    while (enum.hasMoreElements())
    {
       String sourcefile = ""+enum.nextElement();  
       
       StringBuffer buf = new StringBuffer();
       buf.append(compiler+" ");
       appendCompileOptions(buf);
       
       
       buf.append(""+sourcefile+" ");
       buf.append("-o "+sourcefile+".o");
       
       java.io.File src = new java.io.File( sourcefile );
       java.io.File obj = new java.io.File( sourcefile+".o" );
       
       if ( (obj.exists()) && (obj.lastModified()>src.lastModified()) )
       {
         //System.out.println("No need to compile "+sourcefile);
       }
       else
       {
         //System.out.println("Has to compile "+sourcefile);  
         executeCommand(""+buf, "compile");
         //System.out.println("Library->compile  {"+buf+"}");
       }
    }   
    
    enum = resourcefiles.elements();
    while (enum.hasMoreElements())
    {
       String resfile = ""+enum.nextElement();  
       
       String packname = resfile.substring(inputdir.length()+1);
       StringBuffer buf = new StringBuffer();
       buf.append(compiler+" ");
       
       
       buf.append("-c --resource "+packname+" "+resfile+" ");
       buf.append("-o "+resfile+".o");
       
       java.io.File src = new java.io.File( resfile );
       java.io.File obj = new java.io.File( resfile+".o" );
       
       if ( (obj.exists()) && (obj.lastModified()>src.lastModified()) )
       {
         //System.out.println("No need to compile "+sourcefile);
       }
       else
       {
         //System.out.println("Has to compile "+sourcefile);  
         executeCommand(""+buf,"compile");
         //System.out.println("Library->compile  {"+buf+"}");
       }
       
       
    }   
    
    
    // Link phase
    
    StringBuffer buf = new StringBuffer();
    buf.append(compiler+" ");
    appendLinkOptions(buf);
    buf.append("-o "+outputdir+"/"+outputfile+" ");
    
    buf.append("\\\n");
    
    enum = inputfiles.elements();
    while (enum.hasMoreElements())
    {
       String objfile = ""+enum.nextElement()+".o \\\n";  
       buf.append(objfile);
    }   
    
    enum = resourcefiles.elements();
    while (enum.hasMoreElements())
    {
       String objfile = ""+enum.nextElement()+".o \\\n";  
       buf.append(objfile);
    }   
    executeCommand(""+buf,"link");
    //System.out.println("Library->link  {"+buf+"}");
    
  }
  
  private void appendCompileOptions(StringBuffer buf)
  {
    Enumeration enum = compileoptions.elements();
    while (enum.hasMoreElements())
    {
      buf.append(""+enum.nextElement()+" ");   
    }
  }
  
  private void appendLinkOptions(StringBuffer buf)
  {
    Enumeration enum = linkoptions.elements();
    while (enum.hasMoreElements())
    {
      buf.append(""+enum.nextElement()+" ");   
    }
  }
  
  private void executeCommand(String cmd, String phase)
      throws NativeCompilationException, NativeLinkException
  {
    int exitcode = 0; 
    String res = "";
    try
    {
      System.out.print(".");  
      //System.out.println("run command -> \n\n"+cmd+"\n\n");
      
      Process p_gcj = null;
      
      if (phase.equalsIgnoreCase("link"))
      {
        com.rimfaxe.util.RimfaxeFile file = new com.rimfaxe.util.RimfaxeFile("work/"+phase+".sh");
        file.rewrite(""+cmd);
      
        p_gcj = Runtime.getRuntime().exec("sh work/"+phase+".sh");
      }
      else
      {
        p_gcj = Runtime.getRuntime().exec(cmd);  
      }
      
      
      java.io.InputStream is = p_gcj.getErrorStream();
      com.rimfaxe.util.RimfaxeFile rf = new com.rimfaxe.util.RimfaxeFile("uu");
     
      exitcode = p_gcj.waitFor();
      
      
      res = rf.read(is);
      
      
    }
    catch (Exception e) 
    { 
      // TODO
    }
    
    if (exitcode!=0) 
    {
        StringBuffer error_msg = new StringBuffer();   
        error_msg.append("\n\nrun command -> \n\n"+cmd+"\n\n"); 
        error_msg.append("Compile phase : "+phase+"\n");
        error_msg.append("Errors compiling sources, bailing out...\n");
        error_msg.append("GCJ exitcode = "+exitcode+"\n");
        error_msg.append("GCJ outputs : \n"+res+"\n");
        
        System.out.println("Compile phase : "+phase);
        
        if (phase.equalsIgnoreCase("compile"))
          throw new NativeCompilationException(""+error_msg);
        if (phase.equalsIgnoreCase("link"))
          throw new NativeLinkException(""+error_msg);
    }
    
  }
  
  private void appendClasspath(StringBuffer buf, Vector files)
  {
    buf.append("--classpath=");
    Enumeration enum = files.elements();
    while (enum.hasMoreElements())
    {
       String file = ""+enum.nextElement();  
       buf.append(file);
       if (enum.hasMoreElements())
         buf.append(":");
    }
    buf.append(" ");
  }
}
