/*
 * Server.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.seda;

/**
 *
 * @author Lars Andersen
 */
public class Server
{
  private String identifier = "";
  
  seda.sandStorm.main.SandstormConfig sandstorm_config = null;
  seda.sandStorm.internal.sandStormMgr sandStormMgr = null;
  
  com.rimfaxe.webserver.ServerPort serverport = null;
  
  /** Creates a new instance of Server */
  public Server() 
  {
  }
    
  public void emitTraces()
  {
    System.out.println("Server listening on port "+serverport.getPort()+" using "+serverport.getProtocol()+" protocol");   
  }
  
  public void initialize (ServerHandlerManager shm,
			  String identifier,
			  com.rimfaxe.webserver.ServerPort serverport,
                          com.rimfaxe.webserver.Configuration conf) 
  {
     try
     { 
       this.identifier = identifier;
       this.serverport = serverport;
       sandstorm_config = new seda.sandStorm.main.SandstormConfig();
     }
     catch (Exception e) 
     {
       System.out.println("com.rimfaxe.webserver.seda.Server : "+e);  
     }
     
     try
     { 
       String[] httprecv_initargs = {"specialURL=/RWS/STATS","ServerPort="+serverport.getPort(),"httpPort="+serverport.getPort()};
       sandstorm_config.addStage("HttpRecv", "seda.apps.Haboob.http.HttpRecv", httprecv_initargs);
     
       String[] httpsend_initargs = {"ServerPort="+serverport.getPort()};
       sandstorm_config.addStage("HttpSend", "seda.apps.Haboob.http.HttpSend", httpsend_initargs);
      
       String[] cachestage_initargs = {"defaultURL=index.html","rootDir=/","specialURL=/RWS/STATS","maxCacheSize="+conf.getCacheSize()};
       sandstorm_config.addStage("CacheStage", "com.rimfaxe.webserver.seda.PageCacheSized", cachestage_initargs);
     
       String[] dynamichttp_initargs = {};
       sandstorm_config.addStage("DynamicHttp", "com.rimfaxe.webserver.seda.DynamicHttp", dynamichttp_initargs);
       
       sandStormMgr = new seda.sandStorm.internal.sandStormMgr(sandstorm_config);
     }
     catch (Exception e) 
     {
       System.out.println("com.rimfaxe.webserver.seda.Server : "+e);   
       
     }
  }
  
  public String getIdentifier() { return identifier; }
  
  public synchronized void shutdown () 
  {
    try
    {   
       sandStormMgr.stop();	
    }
    catch (Exception e) 
    {
       System.out.println("com.rimfaxe.webserver.seda.Server : "+e);  
       
    }
  }
  
  /**
     * start the server
     * it will than startup, and finally run in its own thread.
     */
  public void startup()
  {
    try
    {   
       sandStormMgr.start();	
    }
    catch (Exception e) 
    {
       System.out.println("com.rimfaxe.webserver.seda.Server : "+e);  
    }
  }
    
  
  
  
}
