/*
 * RimfaxeFilterChain.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.servletapi;

import seda.sandStorm.lib.http.httpRequest;
import com.rimfaxe.webserver.seda.SedaHttpResponse;



import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.ServletException;

import com.rimfaxe.webserver.*;
import com.rimfaxe.webserver.servletapi.*;
import com.rimfaxe.webserver.servletapi.ServletFactory;

import java.io.IOException;
import java.util.*;

/**
 *
 * @author  Lars Andersen
 */
public class RimfaxeFilterChain implements javax.servlet.FilterChain 
{
    ServletFactory factory = null; 
    Configuration conf = null; 
  
    
    ServletWrapper servletwrapper = null;
    javax.servlet.Servlet servlet = null;
    WebContext webcontext = null;
    httpRequest request = null;
    VirtualHost virtualhost = null;
    com.rimfaxe.webserver.webapp.Servlet servlet_desc = null;
    
    Vector filters = new Vector();
    int current = 0;
    String servletname = null;
    
    /** Creates a new instance of RimfaxeFilterChain */
    public RimfaxeFilterChain(httpRequest req, VirtualHost vh, WebContext webcontext,Configuration conf, ServletFactory factory) 
    {
      this.request = req;
      this.virtualhost = vh;
      this.webcontext = webcontext;
      this.conf = conf;
      this.factory = factory;
      
      servletname = webcontext.getServletName(request.getURL());
      
      servlet_desc = webcontext.getWebApp().getServlet(servletname);
      if (servlet_desc==null) servlet_desc = conf.getWebApp().getServlet(servletname);
      
      servletwrapper = factory.getWrapper(virtualhost, webcontext, servletname, servlet_desc.getClassName() , 
                                          request.getURL() , servlet_desc);
      
      servlet = servletwrapper.getServlet();
      
      filters = webcontext.getFilters(request.getURL());
    }
    
    
    /** Creates a new instance of RimfaxeFilterChain */
    public RimfaxeFilterChain(String servletname, String urlpath, VirtualHost vh, WebContext webcontext, Configuration conf) 
    {
      this.virtualhost = vh;
      this.webcontext = webcontext;
      this.conf = conf;
       
      this.servletname = servletname;
      this.factory = com.rimfaxe.webserver.servletapi.ServletFactory.getInstance();
      
      servlet_desc = webcontext.getWebApp().getServlet(servletname);
      if (servlet_desc==null) servlet_desc = conf.getWebApp().getServlet(servletname);
      
      //System.out.println("RimfaxeFilterChain, get ServletWrapper");
      
      
      
      
      servletwrapper = factory.getWrapper(virtualhost, webcontext, servletname, servlet_desc.getClassName(), urlpath , servlet_desc);
      
      //System.out.println("RimfaxeFilterChain, get Servlet"); 
      servlet = servletwrapper.getServlet();
      
      //System.out.println("RimfaxeFilterChain, get filters"); 
      filters = webcontext.getFilters(urlpath);
    }
    
    
    public void doFilter(javax.servlet.ServletRequest servletRequest, javax.servlet.ServletResponse servletResponse) 
        throws java.io.IOException, javax.servlet.ServletException 
    {
      //System.out.println("RimfaxeFilterChain, doFilter");  
      if ( current<filters.size() )
      {
         javax.servlet.Filter active_filter = (javax.servlet.Filter) filters.get(current);
         
         //System.out.println("RimfaxeFilterChain, run filter #"+current); 
         //System.out.println("RimfaxeFilterChain, run filter "+active_filter.getClass().getName()); 
         
         current++;
         active_filter.doFilter(servletRequest, servletResponse, this);
         
         try
         {
           //System.out.println("RimfaxeFilterChain, flush buffer");  
           servletResponse.flushBuffer();
         }
         catch (Exception e)
         {
           // ignore exceptions when flushing  
         }
      }
      else
      {
        //System.out.println("RimfaxeFilterChain, run servlet");    
        doService(servletRequest, servletResponse); 
        //System.out.println("RimfaxeFilterChain, run servlet - done"); 
      }
    }
    
    public void doService(javax.servlet.ServletRequest servletRequest, javax.servlet.ServletResponse servletResponse) 
        throws java.io.IOException, javax.servlet.ServletException 
    {
       
       //System.out.println("RimfaxeFilterChain, run servlet");    
       servletwrapper.service(servletRequest, servletResponse); 
    }
    
    public SedaHttpResponse runservice()
        throws ServletException, IOException
    {
      RimfaxeHttpServletResponse jRes = null;
      RimfaxeHttpServletRequest jReq = null;
     
      //System.out.println("RimfaxeFilterChain, create new Response Object"); 
      jRes = new RimfaxeHttpServletResponse(request,webcontext);
            
      //System.out.println("RimfaxeFilterChain, create new Request Object"); 
      
      jReq = new RimfaxeHttpServletRequest(servlet,request,jRes,webcontext);
     
      doFilter(jReq, jRes);
	
      //System.out.println("RimfaxeFilterChain, return reply"); 
      return jRes.getSedaResponse();
    }
}
