/*
 * RimfaxeHttpServletResponse.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.servletapi;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import com.rimfaxe.webserver.*;
import com.rimfaxe.webserver.servletapi.session.HttpSetCookie;
import com.rimfaxe.webserver.seda.SedaHttpResponse;

import seda.sandStorm.lib.http.httpRequest;
import seda.sandStorm.core.BufferElement;

//import com.rimfaxe.server.www.mime.*;
//import com.rimfaxe.server.www.http.*;
//import com.rimfaxe.server.html.HtmlGenerator;


public class RimfaxeHttpServletResponse implements HttpServletResponse 
{

    public final static String CHARSET_PARAMETER = "charset";
    public final static int DEFAULT_BUFFER_SIZE   = 8 * 1024; // 8KB buffer
    public final static int MIN_BUFFER_SIZE       = 4 * 1024; // 4KB buffer
    private final static int STATE_INITIAL        = 0;
    private final static int STATE_HEADERS_DONE   = 1;
    private final static int STATE_ALL_DONE       = 2;
    private final static int STREAM_STATE_INITIAL = 0;
    private final static int STREAM_WRITER_USED   = 1;
    private final static int OUTPUT_STREAM_USED   = 2;

    private int stream_state = STREAM_STATE_INITIAL;

    private RimfaxeServletOutputStream output = null;
    private PrintWriter               writer = null;

    //private MimeTypeFormatException setContentTypeException = null;

    // servlet has set a fixed content length or not, 
    // and cut (see flushStream) any data which are too much here...
    private final static int CALC_CONTENT_LENGTH = -1; 
    private              int fixedContentLength  = CALC_CONTENT_LENGTH;

    // Our Locale
    protected Locale locale;

    
    protected RimfaxeHttpServletRequest jrequest = null;

    protected int buffer_size;

    protected void setServletRequest(RimfaxeHttpServletRequest jrequest) 
    {
	this.jrequest = jrequest;
    }

    public static final String INCLUDED = "com.rimfaxe.webserver.servlet.included";
    public static final String STREAM = "com.rimfaxe.webserver.servlet.stream";
    public static final String MONITOR = "com.rimfaxe.webserver.servlet.monitor";

    int   state = STATE_INITIAL;
    
    httpRequest request = null;

    String content_type = null; //"text/html";
    
    Hashtable headers = new Hashtable();
    
    int status = 200;
    
    WebContext webcontext;
    
    Vector setCookies = new Vector();
    
    /**
     * Sets the content length for this response. 
     * @param len - the content length 
     */
    public void setContentLength(int i) 
    {
	fixedContentLength = i;
        setHeader("Content-Length", ""+i);
        
    }

   
    public void setContentType(String spec) 
    {
       content_type = spec;	
       setHeader("Content-Type", spec);
    }

    protected boolean isStreamObtained() 
    {
	return (stream_state != STREAM_STATE_INITIAL);
    }

    
    public synchronized ServletOutputStream getOutputStream()
	throws IOException
    {
        
        //System.out.println("RimfaxeHttpServletResponse -> Get Output Stream"); 
	if (stream_state == STREAM_WRITER_USED)
        {
            System.out.println("RimfaxeHttpServletResponse -> Writer used");  
	    throw new IllegalStateException("Writer used");
        }
	stream_state = OUTPUT_STREAM_USED;
	return getRimfaxeOutputStream(false);
    }

    protected ServletOutputStream getRimfaxeOutputStream(boolean writerUsed)
	throws IOException
    {
	if ( output != null )
	    return output;

	output = new RimfaxeServletOutputStream(this,writerUsed);
	return output;
    }

    /**
     * Sets the status code and message for this response. If the field had
     * already been set, the new value overwrites the previous one. The message
     * is sent as the body of an HTML page, which is returned to the user to
     * describe the problem. The page is sent with a default HTML header; the
     * message is enclosed in simple body tags (<body></body>).
     * @param i - the status code 
     * @param reason - the status message
     * @deprecated since jsdk2.1
     */
    public void setStatus(int i, String reason) 
    {
	status = i;
    }

    /**
     * Sets the status code for this response. This method is used to set the
     * return status code when there is no error (for example, for the status
     * codes SC_OK or SC_MOVED_TEMPORARILY). If there is an error, the 
     * sendError method should be used instead.
     * @param i - the status code 
     */
    public void setStatus(int i) 
    {  
       setStatus(i, "no reason");
    }

    /**
     * Adds a field to the response header with the given name and value. If
     * the field had already been set, the new value overwrites the previous
     * one. The containsHeader method can be used to test for the presence of a
     * header before setting its value.
     * @param name - the name of the header field 
     * @param value - the header field's value 
     */
    public void setHeader(String name, String value) 
    {
        //System.out.println("Response, set header "+name+"="+value);
        if (headers.get(name)!=null)
        {
          headers.remove(name);
          headers.put(name,value);
        }
        else
        {
          headers.put(name,value);  
        }
	
    }

    /**
     * Adds a field to the response header with the given name and integer
     * value. If the field had already been set, the new value overwrites the
     * previous one. The containsHeader method can be used to test for the
     * presence of a header before setting its value.
     * @param name - the name of the header field 
     * @param value - the header field's integer value 
     */
    public void setIntHeader(String name, int value) 
    {
	setHeader(name, String.valueOf(value));
    }

    /**
     * Adds a field to the response header with the given name and date-valued
     * field. The date is specified in terms of milliseconds since the epoch. 
     * If the date field had already been set, the new value overwrites the
     * previous one. The containsHeader method can be used to test for the
     * presence of a header before setting its value.
     * @param name - the name of the header field 
     * @param value - the header field's date value 
     */
    public void setDateHeader(String name, long date) 
    {
        com.rimfaxe.util.Calendar cal = new com.rimfaxe.util.Calendar();
        cal.setTimeInMillis(date);
	setHeader(name, cal.getHttpDate());
    }

    public void unsetHeader(String name) 
    {
	headers.remove(name);
    }

    /**
     * Sends an error response to the client using the specified status code
     * and descriptive message. If setStatus has previously been called, it is
     * reset to the error status code. The message is sent as the body of an
     * HTML page, which is returned to the user to describe the problem. The
     * page is sent with a default HTML header; the message is enclosed in
     * simple body tags (<body></body>).
     * @param sc - the status code 
     * @param msg - the detail message 
     * @exception IOException If an I/O error has occurred.
     */
    public void sendError(int i, String msg) 
	throws IOException
    {
        //System.out.println("Send error "+i);
        
        status = i;
        
        if (output==null) this.getRimfaxeOutputStream(false);
        
        com.rimfaxe.webserver.servletapi.standard.ErrorMessage em = new com.rimfaxe.webserver.servletapi.standard.ErrorMessage();
        
        String error_found = "Unknown error";
        String error_msg = msg;
        String uri = request.getURL();
        
        if (i==404)
        {
          error_found = "<b>Error 404</b> - file not found.";  
          error_msg = "<b><i>"+uri+"</i></b> could not be resolved in web context <b><i> "+this.webcontext.getUrlpath()+"</i></b>";
        }
        
	output.print( em.getHTML(uri, error_msg, error_found) );
        
        
    }

    /**
     * Sends an error response to the client using the specified status 
     * code and a default message. 
     * @param sc - the status code 
     * @exception IOException If an I/O error has occurred.
     */
    public void sendError(int i)
        throws IOException
    {
        
        sendError(i,"error");
    }

    /**
     * Sends a temporary redirect response to the client using the specified
     *  redirect location URL. The URL must be absolute (for example, 
     * https://hostname/path/file.html). Relative URLs are not permitted here. 
     * @param url - the redirect location URL 
     * @exception IOException If an I/O error has occurred. 
     */    
    public void sendRedirect(String url)
        throws IOException
    {
	//System.out.println("####### Send redirect to "+url); 
        status = 301;
        
        if (output==null) this.getRimfaxeOutputStream(false);
        
        output.print( "Redirect page" );
        
        String location = url;
        
        
        addHeader("Location", location);
    }

    /**
     * Checks whether the response message header has a field with the
     * specified name. 
     * @param name - the header field name 
     * @return true if the response message header has a field with the 
     * specified name; false otherwise
     */
    public boolean containsHeader(String header) 
    {
	if (headers.get(header)!=null) return true;
        else return false;
    }

    /**
     * Adds the specified cookie to the response. It can be called multiple 
     * times to set more than one cookie. 
     * @param cookie - the Cookie to return to the client 
     */
    public void addCookie(Cookie cookie) 
    {
       this.setCookies.addElement(  convertCookie(cookie)  );       
    }

    private HttpSetCookie convertCookie(Cookie cookie) 
    {
	HttpSetCookie scookie = new HttpSetCookie(true,cookie.getName(),cookie.getValue());
	scookie.setComment(cookie.getComment());
	scookie.setDomain(cookie.getDomain());
	scookie.setMaxAge(cookie.getMaxAge());
	scookie.setPath(cookie.getPath());
	scookie.setSecurity(cookie.getSecure());
	scookie.setVersion(cookie.getVersion());
        //System.out.println("Set-Cookie = "+scookie.toString());
	return scookie;
    }

    /**
     * Encodes the specified URL for use in the sendRedirect method or, if 
     * encoding is not needed, returns the URL unchanged. The implementation 
     * of this method should include the logic to determine whether the 
     * session ID needs to be encoded in the URL.
     * Because the rules for making this determination differ from those used
     * to decide whether to encode a normal link, this method is seperate from
     * the encodeUrl method.
     * <p>All URLs sent to the HttpServletResponse.sendRedirect method should
     * be run through this method. Otherwise, URL rewriting canont be used 
     * with browsers which do not support cookies. 
     * @param url - the url to be encoded. 
     * @return the encoded URL if encoding is needed; the unchanged URL 
     * otherwise. 
     * @deprecated since jsdk2.1
     */
    public String encodeRedirectUrl(String url) 
    {
	try 
        {
	    URL redirect = new URL(url);
	    URL requested = new URL(jrequest.getRequestURI());
	    if ( redirect.getHost().equals(requested.getHost()) &&
		 redirect.getPort() == requested.getPort())
		return encodeUrl(url);
	} 
        catch (MalformedURLException ex) 
        {
	    //error so return url.
	    return url;
	}
	return url;
    }

        /**
     * Encodes the specified URL for use in the sendRedirect method or, if 
     * encoding is not needed, returns the URL unchanged. The implementation 
     * of this method should include the logic to determine whether the 
     * session ID needs to be encoded in the URL.
     * Because the rules for making this determination differ from those used
     * to decide whether to encode a normal link, this method is seperate from
     * the encodeUrl method.
     * <p>All URLs sent to the HttpServletResponse.sendRedirect method should
     * be run through this method. Otherwise, URL rewriting canont be used 
     * with browsers which do not support cookies. 
     * @param url - the url to be encoded. 
     * @return the encoded URL if encoding is needed; the unchanged URL 
     * otherwise. 
     */
    public String encodeRedirectURL(String url) 
    {
	return encodeRedirectUrl(url);
    }

    /**
     * Encodes the specified URL by including the session ID in it, or, if 
     * encoding is not needed, returns the URL unchanged. The implementation of
     * this method should include the logic to determine whether the session ID
     * needs to be encoded in the URL. For example, if the browser supports
     * cookies, or session tracking is turned off, URL encoding is unnecessary.
     * <p>All URLs emitted by a Servlet should be run through this method. 
     * Otherwise, URL rewriting cannot be used with browsers which do not 
     * support cookies.
     * @param url - the url to be encoded. 
     * @return the encoded URL if encoding is needed; the unchanged URL 
     * otherwise. 
     * @deprecated since jsdk2.1
     */
    public String encodeUrl(String url) 
    {
	if (! jrequest.isRequestedSessionIdFromCookie()) 
        {
	    url = url + ((url.indexOf("?") != -1) ? "&" : "?")+
		jrequest.getCookieName()+"="+
		jrequest.getSession(true).getId();
	}
	return url;
    }

    /**
     * Encodes the specified URL by including the session ID in it, or, if 
     * encoding is not needed, returns the URL unchanged. The implementation of
     * this method should include the logic to determine whether the session ID
     * needs to be encoded in the URL. For example, if the browser supports
     * cookies, or session tracking is turned off, URL encoding is unnecessary.
     * <p>All URLs emitted by a Servlet should be run through this method. 
     * Otherwise, URL rewriting cannot be used with browsers which do not 
     * support cookies.
     * @param url - the url to be encoded. 
     * @return the encoded URL if encoding is needed; the unchanged URL 
     * otherwise. 
     */
    public String encodeURL(String url) 
    {
	return encodeUrl(url);
    }

    /**
     * Return the Charset parameter of content type
     * @return A String instance
     */
    public String getCharacterEncoding() 
    {
	return "iso-8859-1";
    }

    /**
     * Returns a print writer for writing formatted text responses. 
     * The MIME type of the response will be modified, if necessary, to
     * reflect the character encoding used, through the charset=... property. 
     * This means that the content type must be set before calling this 
     * method. 
     * @exception UnsupportedEncodingException if no such encoding can be 
     * provided 
     * @exception IllegalStateException if getOutputStream has been called 
     * on this same request.
     * @exception IOException on other errors. 
     */    
    public synchronized java.io.PrintWriter getWriter() 
        throws java.io.IOException
    {
        //System.out.println("RimfaxeHttpServletResponse -> Get Writer");
        
	if (stream_state == OUTPUT_STREAM_USED)
	    throw new IllegalStateException("Output stream used");
	stream_state = STREAM_WRITER_USED;

	if (writer == null) 
        {
	    writer = new PrintWriter(new OutputStreamWriter(getRimfaxeOutputStream(true), getCharacterEncoding()));
	}
        
        //System.out.println("RimfaxeHttpServletResponse -> Get Writer - done");
	return writer;
    }

    /**
     * Flush the output stream.
     * @param close Close the stream if true.
     * @exception IOException if an IO error occurs.
     */
    protected synchronized void flushStream(boolean close) 
	throws IOException
    {
        /*
	if (state == STATE_ALL_DONE) 
        {
	    return;
	}
	int writeLength;

	if (stream_state == OUTPUT_STREAM_USED) {
	    output.flush();
	} else if (stream_state == STREAM_WRITER_USED) {
	    writer.flush();
	    if (close) {
		output.realFlush();
	    } else {
		output.flush();
	    }
	} else {
	    // force flush even if no stream are openned
	    getWriter();
	    writer.flush();
	    output.realFlush();
	}

	if (request.hasState(INCLUDED)) {
	    if (out == null)
		return;

	    if( fixedContentLength != CALC_CONTENT_LENGTH ) {
		writeLength = (out.size() < fixedContentLength) 
		    ? (out.size())
		    : (fixedContentLength);
	    } else {
		writeLength = out.size();
	    }
	    reply.setContentLength(writeLength);
	    
	    OutputStream rout = reply.getOutputStream(false);
	    byte content[] = out.toByteArray();
	    if (close)
		out.close();
	    else 
		out.reset();
	    rout.write(content);
	    rout.flush();
	} else {
	    if ((pout != null) && close) {
		pout.flush();
		pout.close();
	    }
	}
         */
    }

    

    /**
     * Sets the preferred buffer size for the body of the response.  
     * The servlet container will use a buffer at least as large as 
     * the size requested.  The actual buffer size used can be found
     * using <code>getBufferSize</code>.
     *
     * <p>A larger buffer allows more content to be written before anything is
     * actually sent, thus providing the servlet with more time to set
     * appropriate status codes and headers.  A smaller buffer decreases 
     * server memory load and allows the client to start receiving data more
     * quickly.
     *
     * <p>This method must be called before any response body content is
     * written; if content has been written, this method throws an 
     * <code>IllegalStateException</code>.
     * @param size the preferred buffer size
     * @exception IllegalStateException	if this method is called after
     * content has been written
     * @see #getBufferSize
     * @see #flushBuffer
     * @see #isCommitted
     * @see #reset
     */
    public void setBufferSize(int size) 
    {
	if (stream_state != STREAM_STATE_INITIAL) {
	    throw new IllegalStateException("Stream already initialized");
	}
	buffer_size = size < MIN_BUFFER_SIZE ? MIN_BUFFER_SIZE : size;
    }

    /**
     * Returns the actual buffer size used for the response.  If no buffering
     * is used, this method returns 0.
     * @return the actual buffer size used
     * @see #setBufferSize
     * @see #flushBuffer
     * @see #isCommitted
     * @see #reset
     */
    public int getBufferSize() 
    {
	return buffer_size;
    }

    /**
     * Forces any content in the buffer to be written to the client.  A call
     * to this method automatically commits the response, meaning the status 
     * code and headers will be written.
     * @see #setBufferSize
     * @see #getBufferSize
     * @see #isCommitted
     * @see #reset
     */
    public void flushBuffer() 
	throws IOException
    {
        //System.out.println("FLUSH BUFFER");
	if (output != null) 
        {
          if (!output.isClosed())  
          {
	    if (stream_state == STREAM_WRITER_USED) 
            {
		writer.flush();
	    }
            else if (stream_state == OUTPUT_STREAM_USED) 
            {
		output.flush();
	    }
          } 
	    
	}
    }

    /**
     * Returns a boolean indicating if the response has been
     * committed.  A commited response has already had its status 
     * code and headers written.
     * @return a boolean indicating if the response has been
     * committed
     * @see #setBufferSize
     * @see #getBufferSize
     * @see #flushBuffer
     * @see #reset
     */
    public boolean isCommitted() 
    {
	if (output != null) 
        {
	    return output.isCommitted();
	} 
        else 
        {
	    return false;
	}
    }

    /**
     * Clears any data that exists in the buffer as well as the status code and
     * headers.  If the response has been committed, this method throws an 
     * <code>IllegalStateException</code>.
     * @exception IllegalStateException  if the response has already been
     *                                   committed
     * @see #setBufferSize
     * @see #getBufferSize
     * @see #flushBuffer
     * @see #isCommitted
     */
    public void reset() 
    {
	if (output != null) 
        {
	    if (stream_state == STREAM_WRITER_USED) 
            {
		writer.flush();
	    }
	    output.reset();
	}
    }

    /**
     * Sets the locale of the response, setting the headers (including the
     * Content-Type's charset) as appropriate.  This method should be called
     * before a call to {@link #getWriter}.  By default, the response locale
     * is the default locale for the server.
     * @param loc  the locale of the response
     * @see #getLocale
     */
    public void setLocale(Locale locale) 
    {
        System.out.println("Set Locale");
        setHeader("Content-Type","text/html");
	if (locale == null) 
        {
	    return;
	}
	this.locale = locale;
    }

    /**
     * Returns the locale assigned to the response.
     * @see #setLocale
     */
    public Locale getLocale() 
    {
	return locale;
    }

    public void addHeader(String name, String value) 
    {
        
      setHeader(name, value);	
    }

    public void addDateHeader(String name, long date) 
    {
        com.rimfaxe.util.Calendar cal = new com.rimfaxe.util.Calendar();
        cal.setTimeInMillis(date);
	addHeader(name, cal.getHttpDate());  
    }

    public void addIntHeader(String name, int value) 
    {
	addHeader(name, String.valueOf(value));
    }

    public void resetBuffer() {
    }
    
    public SedaHttpResponse getSedaResponse()
    {
      
      
      if (setCookies.size()>0)
      {
        StringBuffer buf = new StringBuffer();  
        Enumeration enum = setCookies.elements();
        while (enum.hasMoreElements())
        {
          HttpSetCookie hsc = (HttpSetCookie) enum.nextElement(); 
          
          if (enum.hasMoreElements()) buf.append( hsc.toString() +", ");
          else buf.append( hsc.toString() );
        }
        headers.put("Set-Cookie", ""+buf);
      }
      
      if (headers.get("Cache-Control")==null)
      {
        String maxage = ""+request.getSpecial("MAX-AGE");
        if (!maxage.equalsIgnoreCase("null"))
        {
          if (!maxage.equalsIgnoreCase("-1")) 
          { 
            headers.put("Cache-Control", "max-age="+maxage); 
          }
        }
      }
      
      
      
      SedaHttpResponse seda = null;
      if (output==null)
      {
        seda = new SedaHttpResponse(status,this.content_type, headers);   
      }
      else
      {
        
        seda = new SedaHttpResponse(status,this.content_type, new BufferElement(output.getBuffer()), headers);
      }
      
      return seda;
    }
    
    RimfaxeHttpServletResponse(httpRequest request, WebContext webcontext) 
    {
	this.request     = request;
        this.webcontext = webcontext;
    }

}
