/*
 * RimfaxeRequestDispatcher.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.servletapi;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.Writer;
import java.io.InputStreamReader;

import java.net.URL;
import java.net.MalformedURLException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.ServletOutputStream;

import com.rimfaxe.webserver.VirtualHost;
import com.rimfaxe.webserver.WebContext;
import com.rimfaxe.webserver.webapp.Servlet;
import com.rimfaxe.webserver.Configuration;

public class RimfaxeRequestDispatcher implements RequestDispatcher 
{
    com.rimfaxe.webserver.servletapi.ServletFactory factory = com.rimfaxe.webserver.servletapi.ServletFactory.getInstance();
    
    VirtualHost virtualhost = null;
    WebContext webcontext = null;
    
    Configuration conf = com.rimfaxe.webserver.ObjectStore.getConfiguration();
    
    public static final String REQUEST_URI_P = "javax.servlet.include.request_uri";
    public static final String CONTEXT_PATH_P = "javax.servlet.include.context_path";
    public static final String SERVLET_PATH_P = "javax.servlet.include.servlet_path";
    public static final String PATH_INFO_P = "javax.servlet.include.path_info";
    public static final String QUERY_STRING_P = "javax.servlet.include.query_string";

    
    private String urlpath = null;
    private String servletname = null;
    
    public void forward(ServletRequest request,	ServletResponse response)
	throws ServletException, IOException
    {
        //System.out.println("Forward request");
	
        if (response.getClass().getName().equalsIgnoreCase("com.rimfaxe.webserver.servletapi.RimfaxeHttpServletResponse"))
        {
           RimfaxeHttpServletResponse jres = (RimfaxeHttpServletResponse) response; 
           if (jres.isStreamObtained())
	      throw new IllegalStateException("Can't Forward! OutputStream or "+"Writer has allready been obtained.");
        }
        
        
	
        if (urlpath==null) urlpath = "";
        
        if (servletname.equalsIgnoreCase("unknown"))
        {
          //System.out.println("unknown servlet name, DESTINATION = "+urlpath);  
          
          if (webcontext==null)
          {
            webcontext = virtualhost.getWebContext(urlpath); 
          }
          
          servletname = webcontext.getServletName(urlpath);
          
          //System.out.println("Servlet name, resolved to : "+servletname);  
          
        }
   
        Servlet s = webcontext.getServlet(servletname);
        if (s==null) s = conf.getWebApp().getServlet(servletname);
        
        if (s==null) 
        {
          throw new ServletException("No servlet found");  
        }
        
        String cls = s.getClassName();
    
        RimfaxeFilterChain rfc = new RimfaxeFilterChain(servletname, urlpath, virtualhost, webcontext, conf); 
        rfc.doService(request,response);
        
        
        
    }

    public void include(ServletRequest request,	ServletResponse response)
	throws ServletException, IOException
    {
        //System.out.println("Include from dispatcher, URL="+urlpath); 
        
	if (urlpath==null) urlpath = "";
        
        if (servletname.equalsIgnoreCase("unknown"))
        {
          //System.out.println("unknown servlet name, JSP="+urlpath); 
          
          servletname = webcontext.getServletName(urlpath);
        }
   
        Servlet s = webcontext.getServlet(servletname);
        if (s==null) s = conf.getWebApp().getServlet(servletname);
        
        if (s==null) 
        {
          throw new ServletException("No servlet found");  
        }
        
        String cls = s.getClassName();
    
        
        
        request.setAttribute("javax.servlet.include.request_uri", urlpath);
        request.setAttribute("javax.servlet.include.servlet_path", urlpath);
        request.setAttribute("javax.servlet.include.path_info", urlpath);
        request.setAttribute("javax.servlet.include.context_path",webcontext.getUrlpath());
        
        
        RimfaxeFilterChain rfc = new RimfaxeFilterChain(servletname, urlpath, virtualhost, webcontext, conf);  
        rfc.doService(request,response);
    }

    protected RimfaxeRequestDispatcher(String servletname, String urlpath, VirtualHost vh, WebContext webcontext) 
    {
        //System.out.println("new RimfaxeRequestDispatcher (with webcontext)");
        this.servletname = servletname;
	this.virtualhost  = vh;
	this.urlpath = urlpath;
        this.webcontext = webcontext;
    }

    
    /**
     * Get the appropriate dispatcher
     * @param urlpath the servlet URI
     * @param server the HTTP server
     * 
     * @return the RequestDispatcher
     */
    //public static RequestDispatcher getRequestDispatcher(String urlpath,VirtualHost vh) 
    //{
    //  return new RimfaxeRequestDispatcher("unknown",urlpath, vh, null);        
    //}
    
    /**
     * Get the appropriate dispatcher
     * @param urlpath the servlet URI
     * @param server the HTTP server
     * 
     * @return the RequestDispatcher
     */
    public static RequestDispatcher getRequestDispatcher(String urlpath,VirtualHost vh,WebContext ctxt) 
    {
      return new RimfaxeRequestDispatcher("unknown",urlpath, vh, ctxt);        
    }
    
    
    /**
     * Get the appropriate dispatcher
     * @param servletname the servlet
     * @param server the HTTP server
     * 
     * @return the RequestDispatcher
     */
    public static RequestDispatcher getNamedDispatcher(String servletname, VirtualHost vh,WebContext ctx) 
    {
      return new RimfaxeRequestDispatcher(servletname,null, vh, ctx);        
    }
    
    public String toString() { return "RimfaxeRequestDispatcher"; }
}
