/*
 * RimfaxeServletContext.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.servletapi;

import com.rimfaxe.util.RimfaxeVector;
import java.util.*;

/**
 *
 * @author Lars Andersen
 */
public class RimfaxeServletContext implements javax.servlet.ServletContext
{
    com.rimfaxe.webserver.WebContext webcontext;
    com.rimfaxe.webserver.VirtualHost virtualhost;
    
    Hashtable attributes = new Hashtable();
    Hashtable params = new Hashtable();
    
    /** Creates a new instance of RimfaxeServletContext */
    public RimfaxeServletContext(com.rimfaxe.webserver.WebContext webcontext,com.rimfaxe.webserver.VirtualHost vh) 
    {
      this.webcontext = webcontext; 
      this.virtualhost = vh;
    }
    
    public Object getAttribute(String str) 
    {
      //System.out.println("RimfaxeServletContext,getAttribute("+str+")");  
      return attributes.get(str);
    }
    
    public java.util.Enumeration getAttributeNames() 
    {
      return attributes.keys();
    }
    
    public javax.servlet.ServletContext getContext(String str) 
    {
      javax.servlet.ServletContext ctx = null;
      
      ctx = (javax.servlet.ServletContext) virtualhost.getServletContext(str);
      return ctx;  
    }
    
    public String getInitParameter(String str) 
    {
      return ""+params.get(str);   
    }
    
    public java.util.Enumeration getInitParameterNames() 
    {
      return params.keys(); 
    }
    
    public int getMajorVersion() 
    {
      return 2;  
    }
    
    public String getMimeType(String str) 
    {
      String ext = "unknown";
      //System.out.println("Get mime type for ["+str+"]");  
      StringTokenizer tkz = new StringTokenizer(str,".",false);
      while (tkz.hasMoreTokens()) 
      { 
          ext = tkz.nextToken(); 
      }
      
      //System.out.println("Get mime type extension ["+ext+"]"); 
      return webcontext.getMimeType(ext); 
    }
    
    public int getMinorVersion() 
    {
      return 3;  
    }
    
    public javax.servlet.RequestDispatcher getNamedDispatcher(String str) 
    {
      return RimfaxeRequestDispatcher.getNamedDispatcher(str, virtualhost, webcontext);
    }
    
    public String getRealPath(String str) 
    {
      return webcontext.getRoot()+"/"+str;   
    }
    
    public javax.servlet.RequestDispatcher getRequestDispatcher(String str) 
    {
      return RimfaxeRequestDispatcher.getRequestDispatcher(str, virtualhost,webcontext);
    }
    
    public java.net.URL getResource(String str) 
        throws java.net.MalformedURLException 
    {
       // OBS OBS ikke god med WAR!!!!
       return new java.net.URL("file://"+webcontext.getRoot()+str);
       
    }
    
    public java.io.InputStream getResourceAsStream(String str) 
    {
      byte[] buf = webcontext.getResourceByteArray(str);
      return (java.io.InputStream) new java.io.ByteArrayInputStream(buf); 
    }
    
    public java.util.Set getResourcePaths(String str) 
    {
      HashSet hs = new HashSet();  
      // OBS
      hs.add(str);
      return hs;
    }
    
    public String getServerInfo() 
    {
      return "Rimfaxe Web Server";  
    }
    
    public javax.servlet.Servlet getServlet(String str)   
    {
      //deprecated  
      return null;
    }
    
    public String getServletContextName() 
    {
      //deprecated
      return null;  
    }
    
    public java.util.Enumeration getServletNames() 
    { 
      Vector vec = new Vector();
      return vec.elements();  
    }
    
    public java.util.Enumeration getServlets() 
    {
      Vector vec = new Vector();
      return vec.elements();
    }
    
    public void log(String str) 
    {
      System.out.println(str);  
    }
    
    public void log(Exception exception, String str) 
    {
    }
    
    public void log(String str, Throwable throwable) 
    {
    }
    
    public void removeAttribute(String str) 
    {
      attributes.remove(str);  
    }
    
    public void setAttribute(String str, Object obj) 
    {
      attributes.put(str, obj);  
    }
    
}
