/*
 * RimfaxeServletOutputStream.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.servletapi;

import java.io.*;


import javax.servlet.ServletOutputStream;





public class RimfaxeServletOutputStream extends ServletOutputStream 
{

    ByteArrayOutputStream  out   = new ByteArrayOutputStream();
    
    RimfaxeHttpServletResponse resp;
    boolean committed  = false;
    boolean writerUsed = false;
    boolean closed     = false;
    
    byte ln[] = { (byte)'\r' , (byte)'\n' };

    //byte eof = (byte)'\';
    
    /**
     * Flush the internal buffer.
     */
    private void flushBuffer() 
	throws IOException
    {
	committed = true;
    }

    public void print(int i)
	throws IOException
    {
	write(i);
    }

    public void print(double i)
	throws IOException
    {
	print(Double.toString(i));
    }

    public void print(long l)
	throws IOException
    {
	print(Long.toString(l));
    }

    public void print(String s) 
	throws IOException 
    {
	write(s.getBytes());
    }

    public void println() 
	throws IOException
    {
	write(ln);
    }

    public void println(int i)
	throws IOException
    {
	print(i); println();
    }

    public void println(double i)
	throws IOException
    {
	print(i); println();
    }

    public void println(long l) 
	throws IOException
    {
	print(l); println();
    }

    public void println(String s)
	throws IOException
    {
	print(s); println();
    }

    public void write(int b) 
	throws IOException 
    {
	write((byte)b);
    }

    protected void write(byte b) 
	throws IOException 
    {
      out.write(b);
    }

    public void write(byte b[]) 
	throws IOException 
    {
	write(b, 0, b.length);
    }

    public void write(byte b[], int off, int len) 
	throws IOException 
    {
      out.write(b, off, len);
    }

    public void flush() 
	throws IOException 
    {
       out.flush();
    }

    public void realFlush() 
	throws IOException 
    {
	out.flush();
    }

    public void close() 
	throws IOException 
    {
       
        closed = true;
	out.close();
    }

    public void reset() 
	throws IllegalStateException
    {
	if (committed) 
        {
            System.out.println("IllegalStateException, Response already committed");
	    throw new IllegalStateException("Response already committed");
	}
	out = new ByteArrayOutputStream();
        closed = false;
    }

    public boolean isCommitted() 
    {
	return committed;
    }

    public boolean isClosed() 
    {
	return closed;
    }
    
    public byte[] getBuffer() 
    {
      return out.toByteArray(); 
    }
    
    /**
     * Creates a new buffered RimfaxeServletOutputStream.
     * @param resp the Response
     * @param out The underlying output stream
     * @param bufsize the buffer size
     * @exception IllegalArgumentException if bufsize <= 0.
     */
    RimfaxeServletOutputStream(RimfaxeHttpServletResponse resp,boolean writerUsed)
    {
	this.out        = out;
	this.resp       = resp;
	this.writerUsed = writerUsed;
	this.committed = false;
    }

    
    
}