/*
 * ServletWrapper.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.servletapi;

import java.util.*;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;



import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.SingleThreadModel;
import javax.servlet.UnavailableException;
import javax.naming.directory.DirContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import java.beans.PropertyChangeListener;



import com.rimfaxe.webserver.seda.SedaHttpResponse;
import seda.sandStorm.lib.http.httpRequest;

import com.rimfaxe.webserver.*;

/**
 *
 * @author  Lars Andersen
 */
public class ServletWrapper 
{
  boolean DEBUG = false;
  
  int load_error = 0;
  
  RimfaxeServletConfig servletconfig = null;  
  javax.servlet.Servlet servlet = null;
  String servletclass = "unknown";
  
  /** Creates a new instance of ServletWrapper */
  public ServletWrapper(RimfaxeServletConfig cfg) 
  {
    this.servletconfig = cfg;  
  }
  
  public SedaHttpResponse runservice(httpRequest request, WebContext webcontext)
      throws ServletException, IOException
  {
     RimfaxeHttpServletResponse jRes = null;
     RimfaxeHttpServletRequest jReq = null;
     
     if (DEBUG) System.out.println("ServletWrapper, create new Response Object"); 
     jRes = new RimfaxeHttpServletResponse(request,webcontext);
            
     if (DEBUG) System.out.println("ServletWrapper, create new Request Object"); 
     jReq = new RimfaxeHttpServletRequest(servlet,request,jRes,webcontext);
     
     service(jReq, jRes);
	
     if (DEBUG) System.out.println("ServletWrapper, return reply"); 
     return jRes.getSedaResponse();
  }
  
  public void service(ServletRequest jReq, ServletResponse jRes)
      throws ServletException, IOException
  {
      
     if (DEBUG) System.out.println("ServletWrapper -> "+jRes.getClass().getName());
     if (jRes.getClass().getName().equalsIgnoreCase("com.rimfaxe.webserver.servletapi.RimfaxeHttpServletResponse"))
     {
       if (DEBUG) System.out.println("ServletWrapper, Assign request to response object");   
       com.rimfaxe.webserver.servletapi.RimfaxeHttpServletResponse rresp = (com.rimfaxe.webserver.servletapi.RimfaxeHttpServletResponse) jRes;
       rresp.setServletRequest((com.rimfaxe.webserver.servletapi.RimfaxeHttpServletRequest)jReq);   
     }
      
     if (DEBUG) System.out.println("ServletWrapper, run service in servlet : "+servletclass); 
     
    
     servlet.service(jReq, jRes);
     
     jRes.flushBuffer();
     
     
     
     
  }
  
  
  public javax.servlet.Servlet getServlet() { return servlet; }
  
  /** Return a name string (suitable for use by humans) that describes this
   * Container.  Within the set of child containers belonging to a particular
   * parent, Container names must be unique.
   *
   */
  public String getName() 
  {
    return "RWS Container";  
  }
  
  
  public String convertSlashes(String servletClass)
  {
    StringBuffer buf = new StringBuffer();
    
    StringTokenizer tkz = new StringTokenizer(servletClass,"/",true);
    while (tkz.hasMoreTokens())
    {
      String tmp = tkz.nextToken();
      if (tmp.equalsIgnoreCase("/")) buf.append(".");
      else buf.append(tmp);
    }
    return ""+buf;
  }
  
  /** Set the fully qualified servlet class name for this servlet.
   *
   * @param servletClass Servlet class name
   *
   */
  public void setServletClass(String servletClass) 
  {
    String servletClassConverted = convertSlashes(servletClass);
    
    load_error = 3;  
    if (DEBUG) System.out.println("Set servlet class to "+servletClassConverted);
    try
    {
      this.servletclass = servletClassConverted;  
      servlet = (javax.servlet.Servlet) Class.forName(servletClassConverted).newInstance();
      servlet.init(servletconfig);
      load_error = 0;
    }
    catch (ClassNotFoundException cnfe) 
    {
      System.out.println("ServletWrapper, ClassNotFound "+cnfe);  
      // could not load servlet
      load_error = 1;
      // 
    }
    catch (Exception e) 
    {
      System.out.println("ServletWrapper, Exception -> "+e);    
      // unknown exception
      load_error = 2;
    }
    
    
  }
  
  
  public int getError()
  {
    return load_error;   
  }
  
  
  
}
