/*
 * JSPclass.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.servletapi.jsp;

import java.util.*;

/**
 *
 * @author  Lars Andersen
 */
public class JSPclass 
{
    
  public static final int SUCCESS = 0;  
  public static final int JSP_ERROR = 1;  
  public static final int COMPILE_ERROR = 2;  
  public static final int LINK_ERROR = 3;
  public static final int EXECUTION_ERROR = 4;  
  
  com.rimfaxe.util.Calendar last_modified;
  
  String vhost_name = "";
  String jsp_page = "/index.jsp";
  String lib_name = "";
  
  String java_name = "";
  
  StringBuffer errormsg = new StringBuffer("");
  
  int error = 0;
  
  /** Creates a new instance of JSPclass */
  public JSPclass(String vhost_name, com.rimfaxe.util.Calendar last_modified,
                  String jsp_page) 
  {
    this.vhost_name = vhost_name;  
    this.last_modified = last_modified;
    this.jsp_page = jsp_page;
  }
    
  public String getNativeFileName()
  {
    return "lib"+getServletName()+"_TS_"+last_modified.getTimeInMillisSpecial()+".so";  
  }
  
  public int getError() { return error; }
  
  public void setError(int val) { this.error = val; }
  
  public void setErrorMsg(String val)
  {
    appendErrorMsg("<br> <p style=\"FONT-SIZE: 10pt;margin-left: 10px;margin-right: 10px;\">");  
    appendErrorMsg("<pre style=\"FONT-SIZE: 10pt;margin-left: 10px;margin-right: 10px;\">"+val+"</pre><br>");  
    appendErrorMsg("</p><br>");
  }
  
  public void appendErrorMsg(String val) 
  { 
    errormsg.append(val); 
  }
  
  public void writeStacktrace(Exception e)
  {
    StackTraceElement[] ste = e.getStackTrace();
    
    if (ste.length<=2) return;
    
    appendErrorMsg("<br> <p style=\"FONT-SIZE: 10pt;margin-left: 10px;margin-right: 10px;\">");
   
    appendErrorMsg(""+e+"<br>"); 
    for (int i=0; i<ste.length; i++)
    {
     
       appendErrorMsg(""+ste[i]+"<br>"); 
    }
    appendErrorMsg("</p><br>");
  }
  
  public String getErrorMsg() { return ""+errormsg; }
  
  public String getJspPage() { return jsp_page; }
  
  public String getPackageName()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("nativejsp.");
    buf.append("TS_"+last_modified.getTimeInMillisSpecial()+".");
    buf.append("VH_"+vhost_name);
   
    return ""+buf;  
  }
  
  public String getServletName()
  {
     StringBuffer buf = new StringBuffer();
     buf.append("JSP"+convert(convert(jsp_page,"/","_"),".","_"));
     return ""+buf; 
  }
  
  public String getFullClassName()
  {
    
    return getPackageName()+"."+getServletName();
  }
  
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("nativejsp.");
    buf.append("VH_"+vhost_name+".");
    buf.append(getServletName());
    return ""+buf;  
  }
  
  public String convert(String in, String c_from, String c_to)
  {
    StringBuffer buf = new StringBuffer();  
    StringTokenizer tkz = new StringTokenizer(in,c_from,true);
    while (tkz.hasMoreTokens())
    {
      String tmp = tkz.nextToken();
      if (tmp.equalsIgnoreCase(c_from)) buf.append(c_to);
      else buf.append(tmp);
    }
    return ""+buf;
  }
  
  
  
}
