/*
 * JSPstore.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.servletapi.jsp;

import java.util.*;
import org.apache.naming.resources.Resource;
import com.rimfaxe.webserver.WebContext;
import com.rimfaxe.util.RimfaxeList;
import com.rimfaxe.webserver.compiler.*;
import com.rimfaxe.webserver.compiler.jsp.*;

/**
 *
 * @author  Lars Andersen
 */
public class JSPstore 
{
  WebContext webcontext;
  
  HashMap active_jsp = new HashMap();
  HashMap deferred_jsp = new HashMap();
  RimfaxeList compile_list = new RimfaxeList();
  
  /** Creates a new instance of JSPstore */
  public JSPstore(WebContext webcontext) 
  {
    this.webcontext = webcontext;
  }
   
  public JSPclass lookupJSP(JSPclass jsp_class)
  {
    // Check if this JSP page is readily available
    JSPclass jsp = (JSPclass) active_jsp.get(jsp_class.getFullClassName());
    
    if (jsp!=null) 
    {
      //System.out.println("Already compiled and loaded");
      return jsp;
    }
    
    // Should this JSP page be compiled ?
    
    if (deferred_jsp.get(jsp_class.getFullClassName())==null)
    { 
        
      //Should we wait for a compile to end?  
      if (!compile_list.contains(jsp_class))
      {
        
        // can we load it without compiling?
        // else start compiler
        if (!loadPrecompiledJSP(jsp_class))
        {
          compile_list.add(jsp_class);   
          compileJSP(jsp_class);
        }
        
      }
      else
      {
        //Wait for compile to end 
        waitForCompile(jsp_class);  
      }
      
      
      return jsp_class;
    }
    else
    {
       
      jsp = (JSPclass) deferred_jsp.get(jsp_class.getFullClassName());  
      return jsp;
    }
    
    
  }
  
  public void waitForCompile(JSPclass jsp_class)
  {
       
     int count = 0;
     boolean done = false;
     while (!done)
     {
       count++;
       if (count>30) done = true;
       
         
       if (!compile_list.contains(jsp_class))
           done = true;
       
       try
       {
         Thread.sleep(100);  
       }
       catch (Exception e)
       {
       }
       
     }
     
     // Try to get from active
     // try to get from deferred
     // Mark as unavailable
  }
  
  
  public boolean loadPrecompiledJSP(JSPclass jsp_class)
  {
    com.rimfaxe.webserver.compiler.jsp.JspC jspcompiler = new com.rimfaxe.webserver.compiler.jsp.JspC(jsp_class,this.webcontext);
    
    if (jspcompiler.loadNative())
    {
      //System.out.println("Precompiled loaded -> "+jsp_class.getFullClassName());
      active_jsp.put(jsp_class.getFullClassName(),  jsp_class);
      return true;
    }
    return false;
  }
  
  public void compileJSP(JSPclass jsp_class)
  {
    //System.out.println("Compile -> "+jsp_class.getFullClassName());  
    
    com.rimfaxe.webserver.compiler.jsp.JspC jspcompiler = new com.rimfaxe.webserver.compiler.jsp.JspC(jsp_class,this.webcontext);
   
    try
    {
      System.out.print("Compile -> "+jsp_class.getJspPage()+" ");   
      jspcompiler.compileJSP();  
      active_jsp.put(jsp_class.getFullClassName(),  jsp_class);
      System.out.println(""); 
      
    }
    catch (JspToJavaException e)
    { 
      System.out.println("Compilation Failed, JspToJavaException");   
      jsp_class.setError( jsp_class.JSP_ERROR );  
      jsp_class.setErrorMsg(""+e);
      this.deferred_jsp.put(jsp_class.getFullClassName(),  jsp_class);
      
    }
    catch (NativeCompilationException e)
    { 
      System.out.println("Compilation Failed, NativeCompilationException");   
      jsp_class.setError( jsp_class.COMPILE_ERROR );  
      jsp_class.setErrorMsg(""+e);
      this.deferred_jsp.put(jsp_class.getFullClassName(),  jsp_class);
      
    }
    catch (Exception e)
    { 
        System.out.println("Compilation Failed, Exception"); 
        
        System.out.println(""+e); 
        jsp_class.setError( jsp_class.COMPILE_ERROR );
        //jsp_class.appendErrorMsg("<br>"+e+"<br>");
        jsp_class.writeStacktrace(e); 
        this.deferred_jsp.put(jsp_class.getFullClassName(),  jsp_class);   
        
    }
    
    compile_list.removeObject(jsp_class);
  }
}
