/**
 * HttpSetCookie.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.servletapi.session;

public class HttpSetCookie 
{
    
    protected String name = null;
    
    protected String value = null;
    
    protected String comment = null;
    
    protected String domain = null;
    
    protected int maxage = -1;
    
    protected String path = null;
    
    /**
     * Does this cookie requires special security care by client ?
     */
    protected boolean security = false;
    
    /**
     * This cookie's version.
     */
    protected int version = 1;

    
    public String getComment() 
    {
      return comment;
    }

    
    public void setComment(String comment) 
    {
      this.comment = comment;
    }

    public String getDomain() 
    {
      return domain;
    }

    
    public void setDomain(String domain) 
    {
      this.domain = domain;
    }

 
    public int getMaxAge() 
    {
	return maxage;
    }

  
    public void setMaxAge(int maxage) 
    {
	this.maxage = maxage;
    }

  

    public String getPath() 
    {
	return path;
    }

   

    public void setPath(String path) 
    {
	this.path = path;
    }

  

    public boolean getSecurity() 
    {
	return security;
    }

   

    public void setSecurity(boolean onoff) 
    {
	this.security = onoff;
    }

   

    public int getVersion() 
    {
	return version;
    }

   
    public void setVersion(int version) 
    {
	this.version = version;
    }

   

    public void setName(String name) 
    {
	this.name = name;
    }

   

    public String getName() 
    {
	return name;
    }

  
    public void setValue(String value)
    {
	this.value = value;
    }

    

    public String getValue()
    {
	return value;
    }

    public String toString() 
    {
	String cookie = name+"="+value;
	if (comment != null)
	    cookie = cookie+"; comment="+comment;
	if (maxage != -1)
	    cookie = cookie+"; maxage="+maxage;
	if (domain != null)
	    cookie = cookie+"; domain="+domain;
	if (path != null)
	    cookie = cookie+"; path="+path;
	if (security)
	    cookie = cookie+"; secure";
	return cookie;
    }

    public HttpSetCookie(boolean isValid, String name, String value) 
    {
	this.name  = name;
	this.value = value;
    }

    public HttpSetCookie(String name, String value) 
    {
	this.name  = name;
	this.value = value;
    }

    public HttpSetCookie() 
    {
    }
}


