/*
 * DaemonStat.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.stat;

/**
 *
 * @author  Lars Andersen
 */
public class DaemonStat 
{
  com.rimfaxe.util.Calendar start = new com.rimfaxe.util.Calendar();
  
  long processed = 0;
  long time = 0;
  long bytes = 0;
  
  /** Creates a new instance of DaemonStat */
  public DaemonStat() 
  {
  }
    
  public void statRequest(int clength, long ms)
  {
    processed++;
    time += ms;
    
    bytes += clength/1000+1;
   
  }
  
  public com.rimfaxe.util.Calendar getStart() { return start; }
  public long getProcessed() { return processed; }
  public long getTime() { return time; }
  public long getBytes() { return bytes; }
  
  
  public com.rimfaxe.xml.datamodel.Container getData()
  {
    com.rimfaxe.xml.datamodel.Container data = new com.rimfaxe.xml.datamodel.Container();
 
    //System.out.println("adding literal processed "+processed);
    data.addLiteral("processed",""+processed);
    data.addLiteral("bytes",""+punctuate(""+bytes)+" KB");
    data.addLiteral("time",""+time+" ms");
    data.addLiteral("start",start.getPrintDateSimpleDK()+" "+start.getPrettyTime());
    
    if ( processed!=0)
    {
      long average = time/processed;
      data.addLiteral("average",""+average+" ms");
    }
    
    
    return data;
  }
  
  public String punctuate(String in)
  {
    //System.out.println("in : "+in);  
    StringBuffer buf = new StringBuffer();  
    int first = in.length()%3;
    
    if (first>0)
    {
      String tmp = in.substring(0,first);
      buf.append(tmp);
    }
    
    int offset = first;
    
    while (in.length()>offset)
    {
       if (offset!=0)  buf.append(","); 
       String tmp = in.substring(offset,offset+3);
       buf.append(tmp);
       offset+=3;
    }
    
    //System.out.println("out : "+buf);
    return ""+buf;
  }
  
}
