/*
 * Web.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.webserver.webapp;


import java.util.*;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.rimfaxe.webserver.servletapi.RimfaxeFilterChain;
import com.rimfaxe.util.*;


/**
 *
 * @author  Lars Andersen
 */
public class Web 
{
  HashMap mimetypes = new HashMap(); 
  HashMap servlets = new HashMap();
  HashMap filters = new HashMap();
  
  Hashtable taglibs = new Hashtable();
  
  ArrayList servletmappings = new ArrayList();
  ArrayList staticmappings = new ArrayList();
  ArrayList filtermappings = new ArrayList();
  
  SessionConfig sessionconfig = null;
  WelcomeFileList welcomefilelist = null;
  
  String rootdir = "";
  
  long last_modified = System.currentTimeMillis();
  String filename = "config/default.xml";
  
  /** Creates a new instance of Web */
  public Web() 
  {
    rootdir = "";
    parseXML();
  }
   
  /** Creates a new instance of Web */
  public Web(String filename) 
  {    
    this.filename = filename;
    rootdir = filename.substring(0,filename.length()-16);
    parseXML();
  }
  
  public void parseXML()
  {
    com.rimfaxe.util.RimfaxeFile file = new com.rimfaxe.util.RimfaxeFile(filename);
    String xml = file.read();
 
    java.io.File f = new java.io.File(filename);
    last_modified = f.lastModified();
    
    
    if (file.getError()==0)
    {
      com.rimfaxe.xml.ParserInterface parser = com.rimfaxe.xml.ParserFactory.getInstance().checkoutParser();
      parser.parse(xml,false);
      Document document = parser.getDocument(); 
      traverse(document);      
    } 
  }
  
  /** Creates a new instance of Web */
  //public Web(String xml, boolean validate) 
  //{
  //  com.rimfaxe.xml.ParserInterface parser = com.rimfaxe.xml.ParserFactory.getInstance().checkoutParser();
  //  parser.parse(xml,false);
  //  Document document = parser.getDocument(); 
  //  traverse(document);  
  //}
 
  public void reload()
  {
    java.io.File f = new java.io.File(filename);
    long lm = f.lastModified();
    
    if (lm>last_modified)
    {
      System.out.println("-- web.xml has been changed - reload");  
      parseXML();
    }
    
      
    Enumeration enum = taglibs.elements();
    while (enum.hasMoreElements())
    {
      TagLibraryInfoImpl tl = (TagLibraryInfoImpl) enum.nextElement();
      tl.reload();
    }
  }
  
  public SessionConfig getSessionConfig() { return sessionconfig; }
  
  public Hashtable getTagLibraries()
  {   
    return taglibs;    
  }
  
  public TagLibraryInfoImpl getTagLibrary(String uri)
  {  
    TagLibraryInfoImpl tl = (TagLibraryInfoImpl) taglibs.get(uri);
    return tl;
  }
  
  public String[] getWelcomeFileList()
  {
    if (this.welcomefilelist==null) return null;
    
    return this.welcomefilelist.getArray();
  }
  
  public String getMimeType(String ext)
  {
    MimeMapping t = (MimeMapping) mimetypes.get(ext);
    if (t==null) return null;
    return t.getMimeType();
  }
  
  public ArrayList getServletMappings() { return servletmappings;}
  
  public ArrayList getStaticMappings() { return staticmappings;}
  
  public ArrayList getFilterMappings() { return filtermappings;}
  
  public Servlet getServlet(String name)
  {
    Servlet s = (Servlet) servlets.get(name.toLowerCase());
    return s;
  }
  
  public Filter getFilter(String name)
  {
    
    Filter f = (Filter) filters.get(name.toLowerCase());
    return f;
  }
  
  public Enumeration getServletNames()
  {
     
    Vector vec = new Vector();
    Object[] a = servlets.values().toArray();
    for (int i=0; i<a.length; i++)
    {
      Servlet sdef = (Servlet) a[i];
      vec.addElement( sdef.getKey() );
    }
    return vec.elements();
  }
  
  private String traverse(Node node)
  {
    
    StringBuffer str = new StringBuffer();
    
    if (node == null) 
    {
      return ""; 
    }
    int type = node.getNodeType();
    switch (type) 
    {
      
      case Node.DOCUMENT_NODE: 
      { 
        traverse(((Document)node).getDocumentElement());
        break;
      }
      
      case Node.ELEMENT_NODE: 
      {  
        if (node.getNodeName().equalsIgnoreCase("servlet"))
        {  
           Servlet s = new Servlet(node); 
           servlets.put(s.getKey().toLowerCase(),s);      
        }
        else if (node.getNodeName().equalsIgnoreCase("filter"))
        {  
           Filter f = new Filter(node); 
           filters.put(f.getKey().toLowerCase(),f);      
        }
        else if (node.getNodeName().equalsIgnoreCase("taglib"))
        {  
           TagLibraryInfoImpl tl = new TagLibraryInfoImpl(node,rootdir); 
           taglibs.put(tl.getKey().toLowerCase(),tl);      
        }
        else if (node.getNodeName().equalsIgnoreCase("static-mapping"))
        {
          
           StaticMapping sm = new StaticMapping(node); 
           staticmappings.add(sm);      
        }
        else if (node.getNodeName().equalsIgnoreCase("servlet-mapping"))
        {
          
           ServletMapping sm = new ServletMapping(node); 
           servletmappings.add(sm);      
        }
        else if (node.getNodeName().equalsIgnoreCase("filter-mapping"))
        {
          
           FilterMapping fm = new FilterMapping(node); 
           filtermappings.add(fm);      
        }
        else if (node.getNodeName().equalsIgnoreCase("session-config"))
        {
          
           SessionConfig sc = new SessionConfig(node); 
           sessionconfig = sc;      
        }
        else if (node.getNodeName().equalsIgnoreCase("mime-mapping"))
        {
           MimeMapping mm = new MimeMapping(node); 
           mimetypes.put(mm.getKey(),mm);         
        }
        else if (node.getNodeName().equalsIgnoreCase("welcome-file-list"))
        {
          
           WelcomeFileList wfl = new WelcomeFileList(node); 
           welcomefilelist = wfl;      
        }
        else
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
            }
          }      
        }
        break;
      }
      
      case Node.TEXT_NODE: 
      {
        //if (node instanceof TextImpl) 
        //{ 
          str.append(node.getNodeValue());
        //} 
        break;
      }
    }
    return str.toString();
  }
    
  public String toXML()
  {
    StringBuffer buf = new StringBuffer();   
      
    buf.append("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n\n");
    
    buf.append("<!DOCTYPE web-app \n PUBLIC \"-//Sun Microsystems, Inc.//DTD Web Application 2.3//EN\"  \n  \"http://java.sun.com/dtd/web-app_2_3.dtd\"> \n\n");
    buf.append("<web-app> \n\n");
    
    buf.append("\n");
    
    Object[] objarray = servlets.values().toArray();
    for (int i=0; i<objarray.length; i++)
    {
      Servlet s = (Servlet) objarray[i];
      buf.append(s.toXML()+"\n");
    }
    
    buf.append("\n");
    
    objarray = filters.values().toArray();
    for (int i=0; i<objarray.length; i++)
    {
      Filter f = (Filter) objarray[i];
      buf.append(f.toXML()+"\n");
    }
    
    buf.append("\n");
    
    objarray = filtermappings.toArray();
    for (int i=0; i<objarray.length; i++)
    {
      FilterMapping fm = (FilterMapping) objarray[i];
      buf.append(fm.toXML()+"\n");
    }
    
    buf.append("\n");
    
    objarray = staticmappings.toArray();
    for (int i=0; i<objarray.length; i++)
    {
      StaticMapping sm = (StaticMapping) objarray[i];
      buf.append(sm.toXML()+"\n");
    }
    
    buf.append("\n");
    
    objarray = servletmappings.toArray();
    for (int i=0; i<objarray.length; i++)
    {
      ServletMapping sm = (ServletMapping) objarray[i];
      buf.append(sm.toXML()+"\n");
    }
    
    if (sessionconfig!=null) buf.append("\n"+sessionconfig.toXML()+"\n");
    
    objarray = mimetypes.values().toArray();
    for (int i=0; i<objarray.length; i++)
    {
      MimeMapping mm = (MimeMapping) objarray[i];
      buf.append(mm.toXML());
    }
    
    if (welcomefilelist!=null) buf.append("\n"+welcomefilelist.toXML()+"\n");
    
    buf.append("</web-app>\n\n");
  
    
    return ""+buf;   
  }
  
  
  
}
