/*
 * Parser.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
package com.rimfaxe.xml;

import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;



import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
/** 
 *
 * @author  Lars Andersen
 */
public class XercesParser implements ParserInterface 
{

  String XMLcontent = "";
  Document document = null;
  
  StringBuffer errorlog = new StringBuffer();
  StringBuffer error = new StringBuffer();
  
  /** Creates new Parser */
  public XercesParser() 
  {
  }
  
 
  public void parse(String in) 
  {
    java.io.ByteArrayOutputStream outstream = new java.io.ByteArrayOutputStream();
    System.setErr( new java.io.PrintStream(outstream) );    
    //errorlog.append("com.rimfaxe.xml.Parser");
    
    XMLcontent = in;
    try 
    {
      com.rimfaxe.xml.dom.DOMParserWrapper parser = new com.rimfaxe.xml.xerceswrappers.DOMParser();
      document = parser.parse(in);
      //errorlog.append("Seems to work fine! signal -> [ok]");
    }
    catch (org.xml.sax.SAXParseException spe) 
    {
      //errorlog.append( "org.xml.sax.SAXParseException : "+spe );  
      //errorlog.append("signal -> [exception]");
    }
    catch (org.xml.sax.SAXException se) 
    {
      if (se.getException() != null)
      {
        //errorlog.append( "org.xml.sax.SAXException : "+se.getException() );
        //errorlog.append("signal -> [exception]");
        
      }
      else
      {
        //errorlog.append( "org.xml.sax.SAXException : "+se );  
        //errorlog.append("signal -> [exception]");
        
      }
    }
    catch (Exception e) 
    {
      //errorlog.append( "Exception : "+e );
      //errorlog.append("signal -> [exception]");
    }
    //error.append(outstream.toString());
  }
  
  
  public String getStacktrace(Exception e)
  {
    java.lang.StringBuffer sb = new java.lang.StringBuffer();
    sb.append(e.toString() +"\n\n");
 
    java.io.ByteArrayOutputStream outstream = new java.io.ByteArrayOutputStream();
    e.printStackTrace( new java.io.PrintStream(outstream) );
    sb.append(outstream.toString() );
    return ""+sb;
  }
  
  public String getErrorLog() { return ""+errorlog; }
  
  public String getError() { return ""+error; }
  
 
  public void parse(String in, boolean validating) 
  {
    java.io.ByteArrayOutputStream outstream = new java.io.ByteArrayOutputStream(); 
    //System.setErr( new java.io.PrintStream(outstream) );  
    errorlog.append("com.rimfaxe.xml.Parser(String in, boolean validating)\n");  
    XMLcontent = in;
    try 
    {
      
      if (validating)
      {
        com.rimfaxe.xml.dom.DOMParserWrapper parser = new com.rimfaxe.xml.xerceswrappers.DOMParser();
        document = parser.parse(in);  
      }
      else
      {
        //System.out.println("Parsing with non validating parser");
        com.rimfaxe.xml.dom.DOMParserWrapper parser = new com.rimfaxe.xml.xerceswrappers.NonValidatingDOMParser();
        document = parser.parse(in);  
      }
     
      //errorlog.append("signal -> [ok]\n");
    }
    catch (org.xml.sax.SAXParseException spe) 
    {
      //errorlog.append( "org.xml.sax.SAXParseException : \n"+getStacktrace(spe)+"\n" );  
      errorlog.append("signal -> [exception]\n");
    }
    catch (org.xml.sax.SAXException se) 
    {
      if (se.getException() != null)
      {
        errorlog.append( "org.xml.sax.SAXException : "+getStacktrace(se.getException())+"\n" );  
        errorlog.append("signal -> [exception]\n");
        
      }
      else
      {
        errorlog.append( "org.xml.sax.SAXException : "+getStacktrace(se)+"\n" );  
        errorlog.append("signal -> [exception]\n");
        
      }
    }
    catch (Exception e) 
    {
      
      errorlog.append( "Exception : "+getStacktrace(e)+"\n" );  
      errorlog.append("signal -> [exception]\n");
    }
   
    //error.append(outstream.toString());
  }

  public Document getDocument()
  {
    //System.out.println(errorlog);  
    return document;
  }
}