/*
 * Container.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.xml.datamodel;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


import java.util.*;

/**
 * Contains name/value pairs called literals.
 *
 * @author  Lars Andersen
 */

public class Container extends Object 
{
  com.rimfaxe.util.RimfaxeVector simpletags = new com.rimfaxe.util.RimfaxeVector();  
  String  name      = "Container";
  HashMap literals  = new HashMap();
  HashMap iterators = new HashMap();
  HashMap containers = new HashMap();
  
  String command = "unknown";
  String datakey = "0";
  
  com.rimfaxe.util.Calendar senddate = new com.rimfaxe.util.Calendar();
  
	/**
	 * Creates new Container 
	 *
	 * @return  
	 */   	
  public Container() 
  {}
  
	/**
	 * Creates new Container from xml string
	 *
	 * @param   xml
	 * @return  
	 */    	
  public Container(String xml) 
  {
    simpletags.addElement("br");
    simpletags.addElement("img");
    simpletags.addElement("hr");
    simpletags.addElement("meta");  
    simpletags.addElement("input");  
    
    StringBuffer doc = new StringBuffer();
    
    // !!!!!!!!!!!!!!!!!!!!
    
    doc.append( "DATA.DTD" );  
    doc.append(xml);
    com.rimfaxe.xml.ParserInterface parser = com.rimfaxe.xml.ParserFactory.getInstance().checkoutParser();
    parser.parse(xml,false);
    Document document = parser.getDocument(); 
  }
  
	/**
	 * Creates new Container from XML DOM
	 * Name is taken from XML DOM (id attribute)
	 *
	 * @param   node	XML DOM
	 * @return  
	 */    	
  public Container(Node node) 
  { 
    simpletags.addElement("br");
    simpletags.addElement("img");
    simpletags.addElement("hr");
    simpletags.addElement("meta");  
    simpletags.addElement("input");  
    NodeList children = node.getChildNodes();  
    if (children != null) 
    {
      NamedNodeMap attrs = node.getAttributes();  
      
      
      if (attrs.getNamedItem("id")!=null)
      {
        String id = attrs.getNamedItem("id").getNodeValue();
        this.name = id;
      }
            
      int len = children.getLength();
      for (int i = 0; i < len; i++)  
      {
        traverse(children.item(i)); 
      }
    }       
  }
  
  
	/**
	 * Creates new Container
	 *
	 * @param   name	name of Container
	 * @param   node	XML DOM
	 * @return  
	 */    	
  public Container(String name,Node node) 
  { 
    this.name = name;  
    NodeList children = node.getChildNodes();  
    if (children != null) 
    {
      int len = children.getLength();
      for (int i = 0; i < len; i++)  
      {
        traverse(children.item(i)); 
      }
    }       
  }
  
	/**
	 * Set name of Container
	 *
	 * @param   val	name of Container
	 */   	
  public void setName(String val) { this.name = val; }

	/**
	 * Get name of Container
	 *
	 * @return  name of Container
	 */   	
  public String getName() { return this.name; }

	/**
	 * Add a literal object to Container
	 *
	 * @param   key
	 * @param   val
	 */    	
  public void addObject(String key, Object val)
  {
    literals.put(key.toLowerCase(),val);
  }
  
	/**
	 * Add a literal String to Container
	 *
	 * @param   key
	 * @param   val
	 */    	
  public void addLiteral(String key, String val)
  {
    literals.put(key.toLowerCase(),val);
  }

	/**
	 * Add an iterator to Container
	 *
	 * @param   key
	 * @param   iterator
	 */    	
  public void addIterator(String key, Iterator iterator)
  {
    iterators.put(key.toLowerCase(),iterator);
  }

	/**
	 * Add a Container to Container
	 *
	 * @param   key
	 * @param   container
	 */    	
  public void addContainer(String key, Container container)
  {
    containers.put(key.toLowerCase(),container);
  }

	/**
	 * Get Object from Container
	 *
	 * @param   key
	 * @return  
	 */    	
  public Object getObject(String key)
  {
    return literals.get(key.toLowerCase());
  }  
  
	/**
	 * Get String Literal from Container
	 *
	 * @param   key
	 * @return  
	 */    	
  public String getLiteral(String key)
  {
    String res = (String) literals.get(key.toLowerCase());
    if (res==null) res="";
    return res;
  }  

	/**
	 * Get Iterator from Container
	 *
	 * @param   key
	 * @return  
	 */    	
  public Iterator getIterator(String key)
  {
    Iterator res = (Iterator) iterators.get(key.toLowerCase());
    return res;
  }  

	/**
	 * Get Container from Container
	 *
	 * @param   key
	 * @return  
	 */    	
  public Container getContainer(String key)
  {
    Container res = (Container) containers.get(key.toLowerCase()); 
    return res;
  } 

  
	/**
	 * Traverse Container and add elements to container
	 * Possible elements are:
	 * value -> adds literal with key = attribute("key") and value = children
	 * command -> sets command field
	 * senddate -> sets senddata field
	 * datakey -> sets datakey field
	 * subsection -> adds container
	 * list -> adds iterator
	 *
	 * @param   node
	 * @return  
	 */    	
  private String traverse(Node node)
  {
    
    StringBuffer str = new StringBuffer();
    
    if (node == null) 
    {
       
      return ""; 
    }
    int type = node.getNodeType();
    switch (type) 
    {
      
      case Node.DOCUMENT_NODE: 
      { 
        traverse(((Document)node).getDocumentElement());
        break;
      }
      
      case Node.ELEMENT_NODE: 
      {  
        if (node.getNodeName().equalsIgnoreCase("value"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            NamedNodeMap attrs = node.getAttributes();  
            String id = attrs.getNamedItem("id").getNodeValue();
            int len = children.getLength();
            StringBuffer buf = new StringBuffer();  
            for (int i = 0; i < len; i++)  
            {
              buf.append( traverse(children.item(i)) ); 
            }
            addLiteral(id.toLowerCase(), ""+buf);
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("command"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              command = val;
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("senddate"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              senddate = new com.rimfaxe.util.Calendar(val);
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("datakey"))
        {
          NodeList children = node.getChildNodes();  
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  
            {
              String val =  traverse(children.item(i)); 
              datakey = val;
            }
          }    
        }
        else if (node.getNodeName().equalsIgnoreCase("subsection"))
        {
          NamedNodeMap attrs = node.getAttributes();  
          String id = attrs.getNamedItem("id").getNodeValue();
          addContainer(id.toLowerCase(),new Container(node));
        }
        else if (node.getNodeName().equalsIgnoreCase("list"))
        {
          NamedNodeMap attrs = node.getAttributes();  
          String id = attrs.getNamedItem("id").getNodeValue();
          addIterator(id.toLowerCase(),new Iterator(node));
        }
        
        else if (node.getNodeName().equalsIgnoreCase("entity"))
        {
            String id = node.getAttributes().getNamedItem("id").getNodeValue();  
            return "&"+id+";";
        }
        
        else
        {
          StringBuffer buf = new StringBuffer();   
          NodeList children = node.getChildNodes();  
          if (children != null) 
          { // body tag
            int len = children.getLength();
             
            for (int i = 0; i < len; i++)  
            {
              buf.append( traverse(children.item(i)) ); 
            }
            
            //return genericBodyTag( node.getNodeName(), node.getAttributes(), ""+buf );
          }    
          else
          { // simple tag
            //return genericSimpleTag( node.getNodeName(), node.getAttributes());  
          }
          return genericTag( node.getNodeName(), node.getAttributes(), ""+buf);  
        }
        break;
      }
      
      case Node.ENTITY_REFERENCE_NODE: 
      {
        //System.out.println("ENTITY_REFERENCE_NODE  -> name="+node.getNodeName()+" value="+node.getNodeValue());
        str.append( "&"+node.getNodeName()+";");
        break;
      }
      
      case Node.TEXT_NODE: 
      {
        //if (node instanceof TextImpl) 
        //{ 
          str.append(node.getNodeValue());
        //} 
        break;
      }
    }
    return str.toString();
  }
  

  
	/**
	 * Get container as XML
	 *
	 * @return  
	 */   	
  public String toXML()
  {
     StringBuffer buf = new StringBuffer();
     buf.append("<subsection id=\""+name+"\">\n");
     
     java.util.Iterator iter = literals.keySet().iterator();
     while (iter.hasNext())
     {
       String hkey =  ""+iter.next(); 
       buf.append("  <value id=\""+hkey+"\">"+literals.get(hkey)+"</value>\n");
     }
     
     
     iter = containers.keySet().iterator();
     while (iter.hasNext())
     {
       Container c = (Container)  containers.get(iter.next()); 
       buf.append("   "+c.toXML());
     }
     
     iter = iterators.keySet().iterator();
     while (iter.hasNext())
     {
         Iterator i = (Iterator) iterators.get(iter.next());
         buf.append("   "+i.toXML());
     }
     
     buf.append("</subsection>\n");
     return ""+buf;
  }
  
  public String genericTag(String name, NamedNodeMap attrs, String body)
  {
    StringBuffer att = new StringBuffer();
    att.append("<"+name);
    
    for (int i=0; i<attrs.getLength(); i++)
    {
      Node node = attrs.item(i);
      att.append(" "+node.getNodeName()+"=\""+node.getNodeValue()+"\"");
    }
    
    if (simpletags.contains(name))
    {
      att.append("/>");
    }
    else
    { 
      att.append(">");
      att.append(body);
      att.append("</"+name+">\n");
    }
    return att.toString();
  } 
    
  public String genericBodyTag(String name, NamedNodeMap attrs, String body)
  {
    //if (simpletags.contains(name)) return genericSimpleTag(name,attrs);
    
    StringBuffer att = new StringBuffer();
    att.append("<"+name);
    
    for (int i=0; i<attrs.getLength(); i++)
    {
      Node node = attrs.item(i);
      att.append(" "+node.getNodeName()+"=\""+node.getNodeValue()+"\"");
    }
    
    att.append(">");
    att.append(body);
    att.append("</"+name+">\n");
    return att.toString();
  } 
  
  public String genericSimpleTag(String name, NamedNodeMap attrs)
  {
    StringBuffer att = new StringBuffer();
    att.append("<"+name);
    
    for (int i=0; i<attrs.getLength(); i++)
    {
      Node node = attrs.item(i);
      att.append(" "+node.getNodeName()+"=\""+node.getNodeValue()+"\"");
    }
    
    att.append("/>");
    
    return att.toString();
  }
}