/*
 * Condition.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.xml.extractor;

import com.rimfaxe.util.*;
import java.util.*;

/**
 *
 * @author Lars Andersen
 */
public class Condition extends Object 
{
  RimfaxeVector rulevec;

  /** Creates new Condition */
  public Condition(RimfaxeVector rulevec) 
  {
    this.rulevec = rulevec;
  }

  public boolean check(String condition)
  {
    return evaluate(condition);
  }

  protected boolean evaluate(String exp)
  { 
    boolean and = true;
    String token = "";
    com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(exp,"!()",true);
    token = tkz.nextToken();
    if (token.equalsIgnoreCase("("))
    {
      String rightStr = "";
      String leftStr = "";
      
      String str = "";
      int p = 1;
      while (p!=0)
      {
        str = tkz.nextToken();
        
        if (str.equalsIgnoreCase(")"))
        {
          p--;
          if (p!=0) leftStr = leftStr.concat(str);
        }
        else
        {
          if (str.equalsIgnoreCase("(")) p++;
          leftStr = leftStr.concat(str);
        }
     
      }
      if (tkz.hasMoreTokens())
      {
        str = tkz.nextToken();
        if (str.equalsIgnoreCase("and")) and = true;
        else and = false; 
        while (tkz.hasMoreTokens())  { rightStr = rightStr.concat(tkz.nextToken()); }
        if (and) return (evaluate(leftStr) && evaluate(rightStr)); 
        else  return (evaluate(leftStr) || evaluate(rightStr)); 
      }
      return evaluate(leftStr);
    }
    
    if (token.equalsIgnoreCase("true")) return true;
    if (token.equalsIgnoreCase("false")) return false;
    if (token.equalsIgnoreCase("!"))  return !evaluate(tkz.nextToken());
    
    if (token.equalsIgnoreCase("interval"))
    {
      tkz.nextToken(); // (
      String intexp = tkz.nextToken();
      tkz.nextToken(); // )
    
      return interval(intexp);
    }
   
 
    return rulevec.contains(exp);
  }
  
  protected boolean interval(String exp)
  {
    boolean res = false;
    com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(exp,",",false);
    String field = tkz.nextToken();
    Integer from = new Integer(tkz.nextToken());
    Integer to   = new Integer(tkz.nextToken());
    
    String p = rulevec.getPostStr(field,"=");
    
    if (p.equalsIgnoreCase("")) return false;
    if (!onlyNumbers(p)) return false;
    
    Integer actual = new Integer(p);
    if ( (from.intValue()<=actual.intValue()) && (actual.intValue()<=to.intValue()) ) return true;
    
    return false;
  }

  protected String whitespace(String str)
  {
    StringBuffer buf = new StringBuffer();
    com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(str, " \n\t", false);
    while (tkz.hasMoreTokens())
    {
      buf.append(tkz.nextToken());
    }
    return buf.toString();
  }

  protected boolean onlyNumbers(String str)
  {
    boolean only = true;
    com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(str, "0123456789", false);
    while (tkz.hasMoreTokens())
    { 
      String tmp = tkz.nextToken();
      only = false;
    }
    return only;
  }
}