/*
 * Extractor.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.xml.extractor;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


import java.util.*;


/**
 *
 * @author  Lars Andersen
 */
public class Extractor extends Object 
{
  com.rimfaxe.util.RimfaxeVector simpletags = new com.rimfaxe.util.RimfaxeVector();
  String xml = "";
  String template = "";
  Document document = null;
  Stack stack = new Stack();
 
  Condition evaluator;
  
  
  /** Creates new Extractor
   * 
   */   	
  public Extractor() 
  {
    simpletags.addElement("br");
    simpletags.addElement("img");
    simpletags.addElement("hr");
    simpletags.addElement("meta");
    simpletags.addElement("input");
  }

  /** Sets the template for this extractor
   *
   * @param   template	name of template (for caching)
   * @param   str			xml with template content
   */    	
  public void setContent(String template, String str)
  {  
    this.xml = str;
    this.template = template;
    
    com.rimfaxe.xml.ParserInterface parser = com.rimfaxe.xml.ParserFactory.getInstance().checkoutParser();
    parser.parse(xml,false);
    Document document = parser.getDocument(); 
  }



  /** Pushes emaildata on the stack
   * 
   * @param   emaildata
   */   	
  public void addData(com.rimfaxe.xml.datamodel.Container data)
  {
    stack.push(data);
  }
  
  /** Traverse the template
   * 
   *
   * @param   node
   * @return  
   */    	
  public String traverse(Node node)
  {   
    StringBuffer str = new StringBuffer();
    
    int type = 0;
    
    try
    {
      if (node == null) { return ""; }
      type = node.getNodeType();
      
    }
    catch (Exception e)
    {
        com.rimfaxe.util.Log.log("Extractor","Traverse exception 1");
        
    }
    
    switch (type) 
    {
      case Node.DOCUMENT_NODE: 
      { 
        try
        {
          str.append(traverse(((Document)node).getDocumentElement()));
        }
        catch (Exception e)
        {
          com.rimfaxe.util.Log.log("Extractor","Traverse exception 2");
          
        }  
        break;
      }
      
      case Node.ELEMENT_NODE: 
      {
        try
        {
        NodeList children = node.getChildNodes();
        
        if (node.getNodeName().equalsIgnoreCase("include"))
        {
	  // get condition
          Node id = node.getAttributes().getNamedItem("condition");
          String condition = id.getNodeValue();
       
	  // get the section we might want to include
          StringBuffer incsection = new StringBuffer();
          
          if (children != null) 
          {
              int len = children.getLength();
              for (int i = 0; i < len; i++) incsection.append(traverse(children.item(i)));
          }
          
          if (evaluator.check(condition))
          {
            str.append(""+incsection);  
          }
          
          break;
        }
        
        if (node.getNodeName().equalsIgnoreCase("check"))
        {  
          String name = node.getAttributes().getNamedItem("name").getNodeValue();
          String value = node.getAttributes().getNamedItem("value").getNodeValue();
          com.rimfaxe.xml.datamodel.Container container = (com.rimfaxe.xml.datamodel.Container) stack.peek();
          String cval = container.getLiteral(name);
          StringBuffer incsection = new StringBuffer();
         
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  incsection.append(traverse(children.item(i)));
          }
          
          if (value.equalsIgnoreCase(cval))
          {
            str.append(""+incsection);    
          }
          break;
        }
        
        if (node.getNodeName().equalsIgnoreCase("list"))
        {
          Node id = node.getAttributes().getNamedItem("id");
          String val = id.getNodeValue();
          
          resetList(val);
          while (fetchList(val))
          {
            if (children != null) 
            {
              int len = children.getLength();
              for (int i = 0; i < len; i++)  str.append(traverse(children.item(i)));
            }
            endList();
          }
          break;
        }
        
        if (node.getNodeName().equalsIgnoreCase("front"))
        {
        
          Node id = node.getAttributes().getNamedItem("id");
          String val = id.getNodeValue();
          
          resetList(val);
          if (fetchList(val))
          {
            if (children != null) 
            {
              int len = children.getLength();
              for (int i = 0; i < len; i++)  str.append(traverse(children.item(i)));
            }
            endList();
          }
          break;
        }
        
        if (node.getNodeName().equalsIgnoreCase("tail"))
        {
            
          Node id = node.getAttributes().getNamedItem("id");
          String val = id.getNodeValue();
          
          while (fetchList(val))
          {
            if (children != null) 
            {
              int len = children.getLength();
              for (int i = 0; i < len; i++)  str.append(traverse(children.item(i)));
            }
            endList();
          }
          break;
        }
        
        if (node.getNodeName().equalsIgnoreCase("subsection"))
        {
         
          Node id = node.getAttributes().getNamedItem("id");
            
          subsection(id.getNodeValue());
          
          if (children != null) 
          {
            int len = children.getLength();
            for (int i = 0; i < len; i++)  str.append(traverse(children.item(i)));
          }
         
          endsection(); 
          break;
        }
        
        if (node.getNodeName().equalsIgnoreCase("literal"))
        { 
          Node id = node.getAttributes().getNamedItem("id");          
          String lit = literal(id.getNodeValue(),"std");
          str.append(lit);
          break;
        }
        
        if (node.getNodeName().equalsIgnoreCase("dbvalue"))
        {  
          Node id = node.getAttributes().getNamedItem("id");       
          Node level = node.getAttributes().getNamedItem("level");        
          String lit = literal(id.getNodeValue(), level.getNodeValue());
          str.append(lit);
          break;
        }
        
        StringBuffer body = new StringBuffer();
        if (children != null) 
        {
          int len = children.getLength();
          for (int i = 0; i < len; i++)  
          {
            body.append(traverse(children.item(i)));
          }
        }
        boolean simple = false;
        if (children==null) simple = true;
        str.append(processTag(node.getNodeName(),node.getAttributes(),body.toString(),simple));
        
        }
        catch (Exception e)
        {
          com.rimfaxe.util.Log.log("Extractor","Traverse exception 3");
          
        }  
        
        break;
      }
      case Node.ENTITY_REFERENCE_NODE: 
      {
        str.append( processEntity(""+node.getNodeName()));
        break;
      }
      
      case Node.ENTITY_NODE: 
      {
        break;
      }
      
      case Node.TEXT_NODE: 
      {
        try
        {
          
              str.append( cleantext(""+node.getNodeValue()) );
          
        }
        catch (Exception e)
        {
          // TODO
        }  
        break;
      }
      
    }
    

    return str.toString();
  }


  // Methods to be overridden
  public String processEntity(String name) {  return "&"+name+";"; }

  /** Process a Tag
   * This method is overridden by a concrete extractor (f.ex. HTMLExtractor)
   *
   * @param   name	name of tag
   * @param   attrs	tag attributes
   * @param   body	body of tag
   * @return  
   */    	
  public String processTag(String name, NamedNodeMap attrs, String body, boolean simple) {  return body; }

  // default methods

  /** Get a literal
   * 
   *
   * @param   id
   * @param   level
   * @return  
   */    	
  protected String literal(String id, String level)
  {
    com.rimfaxe.xml.datamodel.Container container = (com.rimfaxe.xml.datamodel.Container) stack.peek();
    return container.getLiteral(id);
  }

  /**
   * 
   *
   * @param   id
   */   	
  public void subsection(String id)
  {
    com.rimfaxe.xml.datamodel.Container container = (com.rimfaxe.xml.datamodel.Container) stack.peek();
    com.rimfaxe.xml.datamodel.Container top = container.getContainer(id);
    if (top==null)
    {
      com.rimfaxe.util.Log.log("Couldn't find container ["+id+"] inside ["+container.getName()+"]");  
      stack.push( new com.rimfaxe.xml.datamodel.Container() );
    }
    else
      stack.push( container.getContainer(id) ); 
  }

  /**
   * 
   *
   */  	
  public void endsection()
  {
    stack.pop();
  }

  /**
   * 
   *
   * @param   id
   */   	
  public void resetList(String id)
  {
    com.rimfaxe.xml.datamodel.Container container = (com.rimfaxe.xml.datamodel.Container) stack.peek();
    com.rimfaxe.xml.datamodel.Iterator iterator = container.getIterator(id);
    if (iterator!=null)
      iterator.reset();
  }

  /**
   * 
   *
   * @param   id
   * @return  
   */    	
  public boolean fetchList(String id)
  {
    boolean res = false;
    
    com.rimfaxe.xml.datamodel.Container container = (com.rimfaxe.xml.datamodel.Container) stack.peek();
    com.rimfaxe.xml.datamodel.Iterator iterator = container.getIterator(id);
    if (iterator==null) return false;
    if (iterator.fetch())
    {
      res = true;  
      stack.push(iterator.getContainer());
    }
    return res;
  }

  /**
   * 
   *
   */  	
  public void endList() 
  {
    stack.pop();
  } 

  // util

  /**
   * 
   *
   * @param   in
   * @return  
   */    	
  protected String cleantext(String in)
  {
    if (in.equalsIgnoreCase("null")) return "&nbsp;";
    int countspaces = 0;
    boolean firstChar = false;
    StringBuffer res = new StringBuffer();
    for (int i = 0; i < in.length(); i++)
    {
      switch(in.charAt(i))
      {
        case ' '  : { }
        case '\n' : { }
        case '\r' : { if ( (countspaces==0) && (firstChar==true) ) { res.append(" "); } countspaces++; break; }
        default   : { firstChar = true; countspaces=0; res.append(in.charAt(i)); }
      }
    }
    return res.toString();
  }

 
   

  
  /**
   * 
   *
   * @param   count
   * @return  
   */    	
  protected String blanks(int count)
  {
    StringBuffer res = new StringBuffer();
    for (int i=0; i<count; i++)
    {
      res.append(" ");
    }
    return res.toString();
  }

  public String genericBodyTag(String name, NamedNodeMap attrs, String body)
  {
    //if (simpletags.contains(name)) return genericSimpleTag(name,attrs);
    
    StringBuffer att = new StringBuffer();
    att.append("<"+name);
    
    for (int i=0; i<attrs.getLength(); i++)
    {
      Node node = attrs.item(i);
      att.append(" "+node.getNodeName()+"=\""+node.getNodeValue()+"\"");
    }
    
    att.append(">");
    att.append(body);
    att.append("</"+name+">\n");
    return att.toString();
  } 
  
  public String genericSimpleTag(String name, NamedNodeMap attrs)
  {
    StringBuffer att = new StringBuffer();
    att.append("<"+name);
    
    for (int i=0; i<attrs.getLength(); i++)
    {
      Node node = attrs.item(i);
      att.append(" "+node.getNodeName()+"=\""+node.getNodeValue()+"\"");
    }
    
    att.append("/>");
    
    return att.toString();
  }
  
  /**
   * 
   *
   * @param   name
   * @param   attrs
   * @param   body
   * @return  
   */    	
  public String genericTag(String name, NamedNodeMap attrs, String body)
  {
    StringBuffer att = new StringBuffer();
    att.append("<"+name);
    
    for (int i=0; i<attrs.getLength(); i++)
    {
      Node node = attrs.item(i);
      att.append(" "+node.getNodeName()+"=\""+node.getNodeValue()+"\"");
    }
    
    if (simpletags.contains(name))
    {
      att.append(">");
    }
    else
    { 
      att.append(">");
      att.append(body);
      att.append("</"+name+">\n");
    }
    return att.toString();
  } 



  /** 
   * 
   *
   * @param   in
   * @return  
   */    	
  public int csum(String in)
  {
    int sum = 0;
    for (int i=0; i<in.length(); i++)
    {
      Integer part = new Integer("" + in.charAt(i));
      sum += part.intValue();
    }
    if (sum>9) return csum(""+sum);
    else return sum;
  }
 
  
  
  /** Convert DOS string to UNIX
   * 
   * @param   in	String to convert
   * @return  	Converted string
   */    	
  public String dos2unix(String in)
  {
    StringBuffer buf = new StringBuffer();
    
    com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(in,"\r",false);
    while (tkz.hasMoreTokens()) buf.append(tkz.nextToken());
    return buf.toString();
  }
}