/*
 * HTMLextractor.java
 *
 *
 * Copyright (c) 2003 Rimfaxe ApS  (www.rimfaxe.com).  
 * All rights reserved.
 *
 * This package is written by Lars Andersen <lars@rimfaxe.com> 
 * and licensed by Rimfaxe ApS.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by Rimfaxe ApS
          (www.rimfaxe.com)"
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Rimfaxe", "Rimfaxe Software", "Lars Andersen" and 
 *    "Rimfaxe WebServer" must not be used to endorse or promote products 
 *    derived from this software without prior written permission. For written
 *    permission, please contact info@rimfaxe.com
 *
 * 5. Products derived from this software may not be called "Rimfaxe"
 *    nor may "Rimfaxe" appear in their names without prior written
 *    permission of the Rimfaxe ApS.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.rimfaxe.xml.extractor;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.*;
import com.rimfaxe.util.*;


/**
 *
 * @author  Lars Andersen
 * @version 
 */
public class HTMLextractor extends Extractor 
{  
  String fs = System.getProperty("file.separator");
  
  int alt = 1;
  String body = "";
 
  RimfaxeVector params;
  RimfaxeVector errors = new RimfaxeVector();
  
  /** Creates new HTMLextractor for site*/
  public HTMLextractor(com.rimfaxe.xml.datamodel.Container data, com.rimfaxe.xml.datamodel.Rules rules, 
                       String xmlcontent, String id, RimfaxeVector params) 
  {   
    this.params = params;
    if (params.contains("error")) 
    {
       String error = ((com.rimfaxe.http.Parameter)params.find("error")).getValue();
       com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(error,",",false);
       while (tkz.hasMoreTokens())
       {
         errors.addElement( tkz.nextToken() );  
       }
    }
    
    StringBuffer content = new StringBuffer();
    
    //String DTDdir = com.rimfaxe.balancer.ObjectStore.getConfiguration().getHome()+"/admin/dtd/";
    //com.rimfaxe.util.File file = new com.rimfaxe.util.File(DTDdir+"html.dtd");
    //System.out.println("Read DTD -> "+DTDdir+"html.dtd");
    //content.append(file.read());
    
    content.append("<?xml version='1.0'?>\n\n");
    content.append(xmlcontent);
    
    setContent(id,content.toString());
    
    evaluator = new Condition(rules.makeRuleVec());
    
    addData(data);
    
    
    if (document==null)  com.rimfaxe.util.Log.log("Document==null");
    body = traverse(document);
  }
  
 

  public String processTag(String name, NamedNodeMap attrs, String body, boolean simple)
  {
    if (name.equalsIgnoreCase("document")) { return "" +body; }
    if (name.equalsIgnoreCase("title")) { return "<title> \n" +body+ "\n</title>\n"; }    
    if (name.equalsIgnoreCase("list_even")) 
    {
      if (alt==2) { alt = 1;  return ""+body; }
      else        { alt = 2;  return ""; }
    } 
    if (name.equalsIgnoreCase("list_odd")) 
    {
      if (alt==1) { return ""+body; }
      else        { return ""; }
    }
    if (name.equalsIgnoreCase("entity")) 
    {
      String id = ""+attrs.getNamedItem("id").getNodeValue();     
      return "&"+id+";";
    }   
    if (name.equalsIgnoreCase("a")) return anchor(attrs, body);
    if (name.equalsIgnoreCase("input")) return inputfield(attrs);
    if (name.equalsIgnoreCase("form")) return form(attrs, body);
    if (name.equalsIgnoreCase("space")) return "&nbsp;";
    if (name.equalsIgnoreCase("newline")) return "\n";
    
    if (name.equalsIgnoreCase("fileseparator")) return fs;
    
    if (name.equalsIgnoreCase("amp")) return "&amp;";
    if (name.equalsIgnoreCase("br")) return "<br> \n";
    if (name.equalsIgnoreCase("comment")) return "<!--"+body+"-->";
    
    if (name.equalsIgnoreCase("errorblock")) 
    {
      String id = ""+attrs.getNamedItem("id").getNodeValue();   
      if (errors.contains(id)) return body;
      else return "";
    }
    
    if (name.equalsIgnoreCase("errormessage")) 
    {
        
      String id = ""+attrs.getNamedItem("id").getNodeValue();   
      
      if (params.contains("error_"+id)) 
      {
        try
        {
            
          String msg = ((com.rimfaxe.http.Parameter)params.find("error_"+id)).getValue();
          
          return msg;
        }
        catch (NullPointerException npe) 
        {
          
        }
      }
      return "";
    }
    
    return genericTag(name, attrs, body);
  }

  public String inputfield(NamedNodeMap attrs)
  {
    StringBuffer buf = new StringBuffer();
    
    buf.append("<input ");
    
    Node type_node = attrs.getNamedItem("type");
    Node name_node = attrs.getNamedItem("name");
    Node checked_node = attrs.getNamedItem("checked");
    Node value_node = attrs.getNamedItem("value");
    
    
    String type="unknown";
    if (type_node!=null)
    {
       type = attrs.getNamedItem("type").getNodeValue();  
       buf.append("type=\""+type+"\" ");
    }
    
    String name = "unknown";
    if (name_node!=null)
    {  
       name = attrs.getNamedItem("name").getNodeValue(); 
       //System.out.println("input box -> "+name); 
       buf.append("name=\""+name+"\" ");
    }
    
    String value = "unknown";
    if (value_node!=null)
    {
       value = attrs.getNamedItem("value").getNodeValue();  
       buf.append("value=\""+value+"\" ");
    }
    else
    {
      if (params.contains(name))
      {
          String val = ((com.rimfaxe.http.Parameter)params.find(name)).getValue();
          buf.append("value=\""+val+"\" ");
      }
    }
    
  
      if (params.contains(name))
      {
        if (type.equalsIgnoreCase("checkbox"))
        {
           buf.append("CHECKED "); 
        }
        else if (type.equalsIgnoreCase("radio"))
        {
           String val = ((com.rimfaxe.http.Parameter)params.find(name)).getValue();
           if (val.equalsIgnoreCase(value))
             buf.append("CHECKED "); 
        }
      }
      else
      {
         if (checked_node!=null)
         {
           //System.out.println("Checked input box");   
           buf.append("CHECKED ");
         } 
      }
    
    
    
    buf.append("/>");
    return ""+buf;
  }
  
  public String anchor(NamedNodeMap attrs, String body)
  {
    StringBuffer res = new StringBuffer();
    
    String href = "";
    String dest = "";
    String action = "";
    
    Node href_node = attrs.getNamedItem("href");
    Node dest_node = attrs.getNamedItem("goto");
    Node action_node = attrs.getNamedItem("action");
 
  
    if (href_node!=null) href = attrs.getNamedItem("href").getNodeValue();
    if (dest_node!=null) dest = attrs.getNamedItem("goto").getNodeValue();
    if (action_node!=null) action = attrs.getNamedItem("action").getNodeValue();
    
    
    if (action.equalsIgnoreCase("params"))
    {
      String params = attrs.getNamedItem("params").getNodeValue(); 
      com.rimfaxe.xml.datamodel.Container c = (com.rimfaxe.xml.datamodel.Container) stack.peek();
      StringBuffer buf = new StringBuffer();
      if (href.indexOf("?")==-1) buf.append(href+"?");
      else buf.append(href+"&");
      com.rimfaxe.util.RimfaxeStringTokenizer tkz = new com.rimfaxe.util.RimfaxeStringTokenizer(params,",",false);
      while (tkz.hasMoreTokens())
      {
        String p = tkz.nextToken();
        buf.append(p+"="+c.getLiteral(p));
        if (tkz.hasMoreTokens()) buf.append("&");
      }
      
      href = ""+buf; 
    }
    
    return "<a style=\"text-decoration: none;\" href=\"/rimfaxe-loadbalancer-admin"+href+"\">"+body+"</a>";
  }

  public String form(NamedNodeMap attrs, String body)
  {
    String action = attrs.getNamedItem("action").getNodeValue();
    String method = attrs.getNamedItem("method").getNodeValue();
 
    return genericTag("form", attrs, body);
  }

  public String getBody() { return body; }  
}