/* 
 * Copyright (c) 2001 by Matt Welsh and The Regents of the University of 
 * California. All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 * Author: Matt Welsh <mdw@cs.berkeley.edu>
 * 
 */


package seda.apps.Haboob;

import java.util.*;

/**
 * This class contains global counters used for recording server
 * statistics.
 */
public class HaboobStats 
{

  public static int numRequests;
  public static int numErrors;

  // Cache statistics
  public static int numStaticRequests;
  public static int numCacheHits;
  public static int numCacheMisses;
  public static int cacheSizeBytes;
  public static int cacheSizeEntries;

  // Profiling
  public static long timeCacheLookup;
  public static int numCacheLookup;
  public static long timeCacheAllocate;
  public static int numCacheAllocate;
  public static long timeCacheReject;
  public static int numCacheReject;
  public static long timeFileRead;
  public static int numFileRead;

  public static int numConnectionsEstablished;
  public static int numConnectionsClosed;

  // Allow different modules to find each other
  //public static seda.apps.Haboob.http.HttpSend httpSend;
  //public static seda.apps.Haboob.http.HttpRecv httpRecv;
  
  public static Hashtable http_sender_table = new Hashtable();
  public static Hashtable http_receiver_table = new Hashtable();
  
  
  public static seda.apps.Haboob.http.HttpRecv getReceiver(String serverport)
  {
    return (seda.apps.Haboob.http.HttpRecv) http_receiver_table.get(serverport);  
  }
  
  public static void  addReceiver(String serverport,seda.apps.Haboob.http.HttpRecv rec)
  {
    System.out.println("HaboobStats, add receiver, serverport="+serverport);  
    http_receiver_table.put(serverport,rec);  
  }
  
  public static seda.apps.Haboob.http.HttpSend getSender(String serverport)
  {
    return (seda.apps.Haboob.http.HttpSend) http_sender_table.get(serverport);  
  }
  
  public static void  addSender(String serverport,seda.apps.Haboob.http.HttpSend send)
  {
    http_sender_table.put(serverport,send);   
  }
}
